<?php
namespace Controller;

use Service\DeleteCascadeService;
use Service\Manager\Mongo\Types\Type;
use Service\PeriodMathExpression\Period;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use \Service\Manager\Mongo\Manager;
use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Moment\Moment;
use Zeedhi\Framework\DTO\Response\Message;
use Cron\CronExpression;
use \Service\DataSource;
use Util\Utilities;

class ImportHistory extends Crud {

    protected $dataSourceName = 'importHistory';
    /** @var \Service\ImportHistory $importHistoryService */
    protected $importHistoryService;
    /** @var Session $sessionManager */
    protected $sessionManager;
    /** @var \Service\DataSource dataSourceService*/
    protected $dataSourceService;

    public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService,
        \Service\ImportHistory $importHistoryService, Session $sessionManager, DataSource $dataSourceService) {

        $mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        parent::__construct($mongoManager, $deleteCascadeService);
        $this->importHistoryService = $importHistoryService;
        $this->sessionManager = $sessionManager;
        $this->dataSourceService = $dataSourceService;
    }

    public function find(Request\Filter $request, Response $response) {
        $filterData = $request->getFilterCriteria();
        $conditions = $filterData->getConditions();
        $this->convertCreateFilterDateToMongoDate($conditions);
        $filterData = new FilterCriteria($this->dataSourceName, $conditions);

        $rows = $this->mongoManager->findBy($filterData)->getRows();
        $this->importHistoryService->prepareHistoryRows($rows);

        $response->addDataSet(new DataSet($this->dataSourceName, $rows));
    }



    public function findQueryRunner(Request\Filter $request, Response $response) {
        $filterData = $request->getFilterCriteria();
        $conditions = $filterData->getConditions();
        $conditions = $this->setFilterDefault($conditions);
        Utilities::convertCreateFilterDateToMongoDate($conditions,'createDate');

        $filterData = new FilterCriteria($this->dataSourceName, $conditions);

        $rows = $this->mongoManager->findBy($filterData)->getRows();

        $this->importHistoryService->prepareHistoryRows($rows);
        $rows = $this->importHistoryService->prepareHistoryRowsQueryRunner($rows);

        $response->addDataSet(new DataSet($this->dataSourceName, $rows));
    }

    public function remove(Request\Filter $request, Response $response) {
        $filterData = $request->getFilterCriteria();
        $conditions = $filterData->getConditions();
        $filterData = new FilterCriteria($this->dataSourceName, $conditions);

        $dataSet = $this->mongoManager->findBy($filterData);
        try {
            $this->mongoManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $dataSet->getRows()));
        }catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function convertCreateFilterDateToMongoDate(&$conditions) {
        foreach ($conditions as &$condition) {
            if ($condition['columnName'] === 'createDate' && is_array($condition['value'])) {
                $condition['value'][0] = Type::getType('date')->convertToDatabaseValue( $condition['value'][0]);
                $condition['value'][1] = Type::getType('date')->convertToDatabaseValue( $condition['value'][1]);
            }

        }
    }

    public function findLastOnesByDataSourceIds(Request $request, Response $response) {
        try {
            $data = array();
            $row = $request->getRow();

            if(!empty($row['dataSourceIds'])) {
                $dataSourceIds = $row['dataSourceIds'];

                $data = $this->importHistoryService->findLastOnesByDataSourceIds($dataSourceIds);
                $this->importHistoryService->prepareHistoryRows($data);
            }

            $response->addDataSet(new DataSet($this->dataSourceName, $data));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getLastImportationDates(Request $request, Response $response) {
        try {
            $row = $request->getRow();
            $indicatorIds = $row["indicatorIds"] ?: [];
            $dataSourceIds = [];
            $filterCriteria = new FilterCriteria('dataSource');
            $filterCriteria->addCondition('hasDataMining', false);
            $dataSourceWithoutDataMining = $this->dataSourceService->findBy($filterCriteria);
            $dataSourceWithoutDataMining = array_column($dataSourceWithoutDataMining, '_id');
            if(!empty($indicatorIds)){
                $filterCriteria = new FilterCriteria("indicator");
                $filterCriteria->addCondition("_id",FilterCriteria::IN,$indicatorIds);
                $dataSources = $this->dataSourceService->findBy($filterCriteria);

                $filterCriteria = new FilterCriteria("indicator");
                $filterCriteria->addCondition("oldId",FilterCriteria::IN,$indicatorIds);
                $dataSourcesByOldId = $this->dataSourceService->findBy($filterCriteria);

                $dataSourceIds = array_values(array_unique(array_merge(array_column($dataSources,"dataSource"),array_column($dataSourcesByOldId,"dataSource"))));
            }
            $organizationId = (string)Utilities::getOrganizationIdForMongo();
            $lastImportDates = $this->importHistoryService->getLastImportDatesByOrganization($organizationId, $dataSourceWithoutDataMining, $dataSourceIds);
            $response->addDataSet(new DataSet('lastImportDates', $lastImportDates));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param array $conditions
     * @return array
     */
    private function setFilterDefault(array $conditions): array
    {
        $baseProcess = Utilities::getValueInFilterDataSourceByName($conditions, 'baseProcess');
        if (empty($baseProcess)) {
            $environment = Utilities::getEnvironment();
            $baseProcess = $environment->getCurrentOrganizationId() . '_' . $environment->getCurrentUserId();
            $conditions[] = [
                "columnName" => "baseProcess",
                "value" => $baseProcess,
                "operator" => "="
            ];
        }
        $createDate = Utilities::getValueInFilterDataSourceByName($conditions, 'createDate');
        if (empty($createDate)) {
            $createDate = array();
            $createDate[] =Period::getDate('-1W');
            $createDate[] =Period::getDate('');

            $conditions[] = [
                "columnName" => "createDate",
                "value" => $createDate,
                "operator" => "BETWEEN"
            ];
        }
        return $conditions;
    }
}
