<?php
namespace Controller;


use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;

class LogImportation {


	public function find(DTO\Request\Filter $request, DTO\Response $response)
	{
		try {
            $conditions = $request->getFilterCriteria()->getConditions();
            /** @var \Service\LogImportation  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\LogImportation');
            $dataSet = $logImportation->find($conditions);
			$response->addDataSet(new DataSet('logImportation', $dataSet));
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
	}

	public function walletClient(DTO\Request\Filter $request, DTO\Response $response)
	{
		try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            /** @var \Service\LogImportation  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\LogImportation');
            $dataSet = $logImportation->walletClient($conditions, $page, $pageSize, $orderBy);
			$response->addDataSet(new DataSet('walletClient', $dataSet));
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
	}

    public function walletClientSave(DTO\Request\DataSet $request, DTO\Response $response)
    {
        $dataSet = $request->getDataSet();
        $dataSet->setDataSourceName('walletClient');
        /** @var \Service\LogImportation  $logImportation */
        $logImportation = InstanceManager::getInstance()->getService('\Service\LogImportation');
        $logImportation->walletClientSave($dataSet);
    }

    public function walletClientRemove(DTO\Request\DataSet $request, DTO\Response $response)
    {
        $dataSet = $request->getDataSet();
        $dataSet->setDataSourceName('walletClient');
        /** @var \Service\LogImportation  $logImportation */
        $logImportation = InstanceManager::getInstance()->getService('\Service\LogImportation');
        $logImportation->walletClientRemove($dataSet);
    }

    public function findFail(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            /** @var \Service\LogImportation  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\LogImportation');
            $dataSet = $logImportation->findFail($conditions);
            $response->addDataSet(new DataSet('logImportationFail', $dataSet));
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function configImportationFail(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            /** @var \Service\NotificationFail  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\NotificationFail');
            $dataSet = $logImportation->findFail($conditions);
            $response->addDataSet(new DataSet('logImportationFail', $dataSet));
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function configImportationFailSave(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $rows = $request->getDataSet()->getRows()[0];
            /** @var \Service\NotificationFail  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\NotificationFail');
            $saas = empty($rows['EMAIL_SAAS']) ? '' : $rows['EMAIL_SAAS'];
            $host = empty($rows['EMAIL_HOST']) ? '' : $rows['EMAIL_HOST'];
            $local = empty($rows['EMAIL_LOCAL']) ? '' : $rows['EMAIL_LOCAL'];
            $logImportation->saveConfigEmail($saas,$host,$local);
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function notificationImportationFail(DTO\Request $request, DTO\Response $response)
    {
        try {
           // $conditions = $request->getFilterCriteria()->getConditions();
            /** @var \Service\LogImportation  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\LogImportation');
            $message =$logImportation->notificationImportationFail();
            $response->addMessage(new Message($message));

        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

	public function importLogImportation(DTO\Request\Filter $request, DTO\Response $response)
	{
		try {
            /** @var \Service\LogImportation  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\LogImportation');
            $logImportation->importLogImportation();
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
	}


}
