<?php
namespace Controller;

use Service\Manager\Mongo\Manager;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Service\PeriodObservation as PeriodObservationService;

class PeriodObservation extends Crud {

	protected $dataSourceName = 'periodObservation';
    protected $mongoManager;
    protected $periodObservationService;

    public function __construct(Manager $mongoManager, PeriodObservationService $periodObservationService){
        $this->mongoManager = $mongoManager;
        $this->periodObservationService = $periodObservationService;
	}

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $filter = new FilterCriteria($this->dataSourceName,$conditions);
        $observations = $this->periodObservationService->find($filter);
        foreach($observations as &$obs){
            if(!empty($obs["period"])){
                $obs["period"]["start"] = $obs["period"]["start"]->format("d/m/Y");
                $obs["period"]["end"] = $obs["period"]["end"]->format("d/m/Y");
            }
        }
        $dataSet = new DataSet('periodObservation', $observations);
        $response->setDataSets(array($dataSet));
    }
    
    public function save(DTO\Request $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows()[0];
        $id = isset($row["_id"]) ? $row["_id"] : null;
        $name = $row["name"];
        $period = $row["period"];
        $indicator = isset($row["indicator"]) ? $row["indicator"] : null;
        $dataSource = isset($row["dataSource"]) ? $row["dataSource"] : null;
        $this->periodObservationService->save($id, $name, $period, $indicator, $dataSource);
    }

    public function delete(DTO\Request $request, DTO\Response $response){
        foreach($request->getRow() as $row){
            $this->periodObservationService->delete($row);
        }
    }
}
