var engine = function (templateManager, eventAggregator, ScreenService, ZHPromise, LibLoginController, localStorageEngine,
                       ZhuAclController, restEngine, requestDataSourceEngine, ApplicationContext,LibUtilities,MessageService) {
    const BEFORE_PROJECT_STARTUP_EVENT = "beforeProjectStartup";
    var self = this;
    localStorage.removeItem('LOGIN_REDIRECT_URL');
    var INTERATIONS = {
        DASHBOARD: "openDashboard",
        INDICATOR_GROUP: "openIndicator",
        INDICATOR_VIEW: "openIndicatorView"
    };
    var promiseMenu = ZHPromise.defer();
    var isShowLoader = true;
    var autResponse;
    var max = 999999999999999999999;
    var nrorg = null;
    getLocalVar = localStorageEngine.getLocalVar;
    setLocalVar = localStorageEngine.setLocalVar;
    eventAggregator.subscribe(BEFORE_PROJECT_STARTUP_EVENT, function () {
        ScreenService.changeLoadingMessage(ScreenService.i18n("Aguarde") + "...");
            startLogin();
    });

    var startLogin = function(){
        var loginDefer = ZHPromise.defer();
        loginPromise = loginDefer.promise;
        eventAggregator.subscribe("OnUpdateCustomizationsRepositories",function(){
            if(isShowLoader){
                ScreenService.showLoader();
            }
        });
        var setAclEventId;
        var uniqueSetAcl = true;
        var setAclFunction = function () {
            if(uniqueSetAcl){
                uniqueSetAcl = false;
                setAcl(setAclEventId);
            }
        };
        setAclEventId = eventAggregator.subscribe('onContainerOpen', setAclFunction);
        LibLoginController.authenticate()
            .then(function(response){
                autResponse = response;
                setLocalVar('ORGANIZATION_ID',response.ORGANIZATION_ID);
                if(getLocalVar("clienteNRORG") && response.ORGANIZATION_ID == '20'){
                    verifyNrOrg(response, loginDefer);
                }else{
                    getLocalVar("clienteNRORG",null);
                    getLocalVar("clienteNRORGPRINCIPAL",null);
                    validateLoginResponse(response, loginDefer);
                }
            })
            .catch(showErrorMessage);
    };

    this.initSessionAcl = function(response, userInfo) {
        var filter = [ {
            name: 'USER',
            value: localStorage.getItem('LOGIN_USER'),
            operator: '='
        }, {
            name: 'IS_LOGGED',
            value: localStorage.getItem('IS_LOGGED') || false,
            operator: '='
        }, {
            name: 'KEEP_CONNECTED',
            value: localStorage.getItem('LOGIN_KEEP_CONNECTED') || 'No',
            operator: '='
        } ];

        requestDataSourceEngine.getDataSource('acl#/lib_startSession', filter)
            .then(function () {
                if(response && userInfo){
                    verifySetHeaderInfoAndBuildUserData(response, userInfo);
                }
            }).catch(self.showErrorMessage);
    };

    var showErrorMessage = function(error) {
        if(typeof error === 'object') {
            if(error.message) {
                ScreenService.showMessage('"Error: ' + error.message + '"', 'error', 'error');
            } else if(error.data.error) {
                ScreenService.showMessage('"Error: ' + error.data.error + '"', 'error', 'error');
            } else {
                ScreenService.showMessage('"Error: ' + error.data + '"', 'error', 'error');
            }
        } else if(error) {
            ScreenService.showMessage('"Error: ' + error + '"', 'error', 'error');
        } else {
            ScreenService.showMessage('Unexpected error occurred.');
        }
    };

    var getCurrentModule = function() {
        var currentModule = window.location.hash.split('/').filter(function (_) {
            return _.length;
        }).pop();
        if (currentModule && currentModule.length > 0) {
            currentModule = currentModule.split(/_|#/)[0];
        }
        return currentModule;
    };

    var getNameOrganization = function (){
        return restEngine.post({
            service: "product#/zmartNameOrg",
            typeRequest: "POST",
            params: {
                "requestType": "FilterData",
                "filter": [
                    {
                        "name": "NRORG",
                        "operator": "=",
                        "value": getLocalVar("clienteNRORG")
                    }
                ]
            },
            timeout: 120000
        });
    };

    var setHeaderInfoAndBuildUserData = function(response){

        var filter = [ {
            'name': 'USER_ID',
            'operator': '=',
            'value': response.USER_ID
        }, {
            'name': 'ORGANIZATION_ID',
            'operator': '=',
            'value': response.ORGANIZATION_ID
        } ];

        requestDataSourceEngine.getDataSource('acl#/findUserData', filter).then(function(data) {
            var userData = data.dataset.findUserData[0];
            var defaultImage = window.modulesPackage ? Config.defaultImageProduct : Config.defaultImageModules;
            var userImage    = userData.IMAGE || defaultImage;
            ScreenService.setHeaderInfo(response.TRADING_NAME);
            ScreenService.buildUserData(response.USER_NAME, [response.COMPANY_NAME],
                function() {
                    if(getCurrentModule() == 'acl') {
                        ScreenService.openWindow('acl#my_profile');
                    } else {
                        LibUtilities.openLinkByMenu('myProfile');
                    }
                },
                userImage);
        });
    };

    var verifySetHeaderInfoAndBuildUserData = function (response) {
        if (getLocalVar("clienteNRORG")) {
            var nmorganizacao = 'Cliente com NRORG '+getLocalVar("clienteNRORG");
            getNameOrganization().then(function (data) {
                if (data &&
                    data.dataset &&
                    data.dataset.nameOrg &&
                    data.dataset.nameOrg[0]) {
                    nmorganizacao = data.dataset.nameOrg[0].NMORGANIZACAO;
                }
                response.COMPANY_NAME = nmorganizacao;
                response.TRADING_NAME = nmorganizacao;
                setHeaderInfoAndBuildUserData(response);
            });
        } else {
            setHeaderInfoAndBuildUserData(response);
        }

    };

    function cleanZmartParametersAndBuildMenuHierarchy() {
        if (!templateManager.wereLoadedMainDashboard) {
            templateManager.wereLoadedMainDashboard = true;

            localStorageEngine.setLocalVar("menuId", "");
            localStorageEngine.setLocalVar("indicatorView", "");
            localStorageEngine.setLocalVar("drilldownParameters", "");
            eventAggregator.subscribe('menuBuilded',promiseMenu.resolve);
            ScreenService.showLoader();
            isShowLoader = false;
            loadAndBuildViewMenuHierarchy();
        }
    }

    var startSession = function(accesProfiles) {
        var filter = [{
            name: 'ACL',
            value: accesProfiles,
            operator: '='
        }];

        if(getLocalVar("clienteNRORG")){
            filter.push(
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar("clienteNRORG")
                }
            );
        }
        if(getLocalVar("clienteNRORGPRINCIPAL")){
            filter.push(
                {
                    "name": "NRORGPRINCIPAL",
                    "operator": "=",
                    "value": getLocalVar("clienteNRORGPRINCIPAL")
                }
            );
        }

        return requestDataSourceEngine.getDataSource('product#/getLoggedUser', filter).then(function (data) {
            cleanZmartParametersAndBuildMenuHierarchy();
            if (ApplicationContext.WebSocketClient) {
                ApplicationContext.WebSocketClient.connect(data.dataset.login.clientNrOrg);
            }
            nrorg = data.dataset.login.organizationId;
            return data && data.dataset ? data.dataset.login : false;
        });

    };

    var buildRequestObj = function(baseUrl, service, params) {
        return {
            "service": service,
            "params": params,
            "typeRequest": "post",
            "baseUrl": baseUrl
        };
    };

    var  verifyNrOrg = function (response, loginDefer){
        return requestDataSourceEngine.getDataSource('product#/findAvailableCliente').then(function (data) {
            var verify = data.dataset.findAvailableCliente.filter(function(response){
               return  response.NRORG == getLocalVar("clienteNRORG");
            });

            if(verify.length > 0){
                validateLoginResponse(response, loginDefer);
            }else{
                setLocalVar("clienteNRORG",null);
                setLocalVar("clienteNRORGPRINCIPAL",null);
                validateLoginResponse(response, loginDefer);
            }
        });
    };

    var setAcl = function (setAclEventId) {
        loginPromise.then(function(){
            var response = autResponse;
            ZhuAclController.setUserRules(response.ACCESS_PROFILE, window.projectId, response.USER_ID, response.ORGANIZATION_ID,
                {
                    canConfigureAcl: response.ACL_ADMINISTRATOR,
                    rules: response.ACL_ADMINISTRATOR ? [] : false,
                    defaultRule: response.ACL_ADMINISTRATOR ? 'A' : false
                }
            );
            templateManager.project.aclDefaultRule = 'A';
            eventAggregator.unSubscribe('onContainerOpen', setAclEventId);
            eventAggregator.publish('loginSuccess', response);
        });
    };

    var validateLoginResponse = function (response, loginDefer) {
        if(response && response.USER_ID) {
            startSession(response.ACCESS_PROFILE).then(function(userInfo) {
                templateManager.project.elementListDefaultRule = 'A';
                var userId = 'Zmart_' + localStorage.getItem('operador');
                if (ApplicationContext.ZhPreferences) {
                    ApplicationContext.ZhPreferences.setUserID(userId);
                } else {
                    ApplicationContext.zhCustomizationsService.registerUser(userId);
                }
                response.TRADING_NAME =  userInfo.organizationName;
                response.COMPANY_NAME =  userInfo.organizationName;
                self.initSessionAcl(response, userInfo);
                localStorageEngine.setLocalVar('isLogged', true);
                localStorageEngine.setLocalVar('username', response.USER_NAME);
                localStorageEngine.setLocalVar('userId', response.USER_ID);
                loginDefer.resolve();
                self.validateCollector();

                return ZHPromise.when();
            }).catch(function(error) {
                loginDefer.reject();
                showErrorMessage(error);
            });
        }
    };

    var ignoreValidateCollector = function(){
        return templateManager.container && templateManager.container.preview;
    };

    // Collector validations
    this.validateCollector = function() {
        if(!ignoreValidateCollector()){
            var clientMaxVersion = null;

            self.getMaxCollectorVersion().then(function(clientInfo){
                clientMaxVersion = clientInfo.maxVersion;

                self.testAllConnections(clientMaxVersion).then(function(response){
                    var testConnections = response.dataset.testConnections;
                    verifyTestResults(clientMaxVersion, testConnections);
                });
            });
        }
    };

    this.getMaxCollectorVersion = function() {
        var fileName = "modules.json";
        return restEngine.requestMetaData(Util.getRightPath(fileName,"json/"), {}).then(function(file){
            return file.data.zeedhiDataCollector;
        });
    };

    this.testAllConnections = function() {
        return restEngine.post("/testAllConnections",
            {
                "requestType": "FilterData",
                "filter": []
            }
        );
    };

    var hasErrorValidVersion = function (version, maxVersion) {
        if (version && maxVersion) {
            var arrayMaxVersion = maxVersion.split('.');
            var arrayVersion = version.split('.');
            var valid = true;

            arrayMaxVersion.some(function (ver, index) {
                if (parseInt(arrayVersion[index]) > parseInt(ver)) {
                    valid = false;
                    return true;
                } else if (parseInt(arrayVersion[index]) < parseInt(ver)) {
                    return true;
                } else if (index == 2) {
                    valid = false;
                }
            });
            return valid;

        }
    };

    this.hasErrorValidVersion = hasErrorValidVersion;

    var verifyTestResults = function (maxVersion, testConnections) {
        var type;
        var time;

        testConnections.forEach(function(connectionTest){
            var connName = connectionTest.connection;
            var testResult = connectionTest.testResult;
            var version = connectionTest.testResult.version;

            time = 10000;
            var messageConfig = {time : 10000};
            var messageCode = '';
            var messageParameters = {
                connName: connName,
                version: version,
                testResult: testResult
            };
            if(connectionTest.hasError) {
                type = 'error';
                messageCode = 'NOT_CONNECTED';
            } else if (hasErrorValidVersion(version,maxVersion)) {
                type = 'alert';
                messageCode = 'UPDATE_VERSION_CONNECTED';
            } else {
                type = 'success';
                messageCode = 'SUCCESS_CONNECTED';
            }
            messageConfig.type = type;

            MessageService.notificationMessageByCode(messageCode, messageParameters, messageConfig);
        });
    };

    var loadAndBuildViewMenuHierarchy = function() {
        return loadViewMenuHierarchy().then(buildViewMenuHierarchy);
    };

    function loadViewMenuHierarchy() {
        return requestDataSourceEngine.getDataSource('product#/view/menuHierarchy', []);
    }

    function removeMenuFix() {
        templateManager.project.groupMenu.some(function (groupMenu) {
            if (groupMenu.isIndicators) {
                return groupMenu.menus.some(function (menu,index) {
                    if (menu.isRemove) {
                        menu.isVisible = false;
                        groupMenu.menus.splice(index,1);
                        return true
                    }
                });
            }
        });
    }

    function buildViewMenuHierarchy(menus) {
        menus = menus.dataset.menuHierarchy;
        var menuTree = [];
        if (Util.isArray(menus) && menus.length) {
            menus = factoryMenus(menus);
            genereteId(menus);
            menuTree = buildMenuTree(menus);
        }
        promiseMenu.then(function(){
            removeMenuFix();
            templateManager.project.addMenus(menuTree);
            setLocalVar('isTeknisa',(nrorg && nrorg == "20"));
            if(nrorg && nrorg == "20"){
                changeClientIsVisible();
            }else{
                templateManager.project.userDataActions = templateManager.project.userDataActions.filter(function(data){return data.isTeknisa != true;});
            }
            if(getLocalVar("clienteNRORG") && getLocalVar("clienteNRORG") != "20"){
                templateManager.project.getMenu("login").isVisible = false;
                templateManager.project.getMenu("acl").isVisible = false;
            }
            isShowLoader = false;
            ScreenService.hideLoader();
        });

    }

    function setlabelActionMenu() {
        templateManager.project.userDataActions.filter(function (data) {
            return data.isTeknisa == true;
        }).forEach(function (action) {
                action.label = ScreenService.i18n(action.labelTeknisa);
                action.icon = action.iconTeknisa;
            }
        );
    }

    function removeActionCliente() {
        templateManager.project.userDataActions = templateManager.project.userDataActions.filter(function (data) {
            return data.name != "changeClient";
        });
    }

    function changeClientIsVisible() {
        requestDataSourceEngine.getDataSource('product#/findAvailableCliente', []).then(function (data) {
            if (data.dataset.findAvailableCliente.length < 2) {
                removeActionCliente();
            }
            setlabelActionMenu();
        }).catch(function () {
            removeActionCliente();
            setlabelActionMenu();
        });
    }

    function factoryMenus(menus) {
        return menus.map(function(menu) {
            var menuItem = factoryMenu(menu._id, menu.name, (menu.parent || "indicators"), menu.order);
            buildDashboardSubmenu(menuItem, menu.hasDashboards);
            buildIndicatorGroupSubmenu(menuItem, menu.indicators);

            return menuItem;
        });
    }

    function buildIndicatorGroupSubmenu(menu, indicators) {
        if (Util.isArray(indicators) && indicators.length) {
            indicators = setIndicatorsWhenItIsNotMenu(indicators);
            var menus = menu.menus.concat(indicators.map(function(indicator) {
                var menuItem = factoryMenu(indicator._id, indicator.name, menu.name, indicator.order);

                if (indicator.isView){
                    addEvent(menuItem, INTERATIONS.INDICATOR_VIEW);
                    menuItem.menus = [];
                } else if (indicator.views.length === 1) {
                    var view = indicator.views[0];
                    menuItem.firstIndicator = factoryMenu(view._id, view.name);
                    addEvent(menuItem, INTERATIONS.INDICATOR_GROUP);
                } else {
                    menuItem.menus = indicator.views.map(function(view) {
                        var viewMenuItem = factoryMenu(view._id, view.name, menuItem.name, view.order);
                        addEvent(viewMenuItem, INTERATIONS.INDICATOR_VIEW);
                        return viewMenuItem;
                    }).sort(sortByOrder);
                }

                return menuItem;
            }));

            menu.menus = menus.sort(sortByOrder);
        }
    }

    function setIndicatorsWhenItIsNotMenu(indicators) {
        var menuIndicators = indicators;
        if (indicators.length === 1 && indicators[0].ignoreFirstMenuLevel) {
            menuIndicators = indicators[0].views.map(function(indicator) {
                indicator.views = [];
                indicator.isView = true;
                return indicator;
            });
        }

        return menuIndicators;
    }

    function factoryMenu(name, label, parent, order) {
        return {
            name: name,
            label: ScreenService.i18n(label),
            parent: parent,
            isVisible: true,
            icon: "project",
            aclIsVisible: true,
            order: (order || null),
            menus: []
        };
    }

    function genereteId(menus) {
        menus.forEach(function(menu){
            if(menu.menus.length > 0){
                genereteId(menu.menus);
            }
            menu.id = Math.floor(Math.random() * max);
        });
    }

    function buildDashboardSubmenu(menu, hasDashboards) {
        if (hasDashboards) {
            var dashboardMenuItem = factoryMenu(menu.name + "_dashboard", "Dashboards", menu.name);
            addEvent(dashboardMenuItem, INTERATIONS.DASHBOARD);

            menu.menus.push(dashboardMenuItem);
        }
    }

    function addEvent(menu, method) {
        menu.events = [{
            name: "MenuOnClick",
            code: "templateManager.interations." + method + "(args.owner)"
        }];
        return menu;
    }

    function buildMenuTree(allMenus, parents, menusTree) {
        if ((Util.isArray(parents) && parents.length)) {
            for (var i in parents) {
                var parent = parents[i];
                var menus = parent.menus.concat(getLevelMenus(allMenus, parent));

                parent.menus = orderMenus(menus);
                buildMenuTree(allMenus, parent.menus, menusTree);
            }
        } else if (!menusTree || menusTree.length === 0) {
            menusTree = orderMenus(getLevelMenus(allMenus, {
                name: "indicators"
            }));
            buildMenuTree(allMenus, menusTree, menusTree);
        }
        return menusTree;
    }

    function getLevelMenus(allMenus, parent) {
        return allMenus.filter(function(menu) {
            return parent.name === menu.parent;
        });
    }

    function orderMenus(menus) {
        return menus.sort(sortByOrder);
    }

    var sortByOrder = function(menuA, menuB) {
        if (menuA.order !== null && (menuB.order === null || menuA.order < menuB.order)) {
            return -1;
        }
        else if (menuB.order !== null && (menuA.order === null || menuB.order < menuA.order)) {
            return 1;
        }
        else if (menuA.order === menuB.order) {
            return 0;
        }
    };

    this.logout = function () {
        ScreenService.confirmMessage('Deseja realmente sair do sistema?', 'ALERT', disconnect);
    };

    var disconnect = function () {
        setLocalVar("clienteNRORG",null);
        setLocalVar("clienteNRORGPRINCIPAL",null);
        LibLoginController.logout();
    };

    eventAggregator.subscribe("onContainerOpen", function (openEvent) {
        var container = openEvent.container;
        if(container && container.label) {
            var updatedTitle = ScreenService.i18n(container.label) + ' - Zmart';
            templateManager.updateTitle(updatedTitle);
        }
    });
};

Configuration(function (ContextRegister) {
    ContextRegister.register('engine', engine);
});

function ClienteService(requestDataSourceEngine) {

    this.loginUpdateSession = function(cdcliente){
        var filters = [
            {name: "OTHER_SESSION_VALUES", value: {'CDCLIENTE':cdcliente}, operator: "="}
        ];
        return requestDataSourceEngine.getDataSource('product#/updateCustomerInSession', filters);
    };

    this.getWidgetClienteJson = function () {
        return {
            "name": "CHANGE_CLIENTE",
            "label": "Trocar de cliente",
            "isVisible": true,
            "showAsModal": true,
            "hasPagination": false,
            "searchWithPagination": false,
            "template": "widget/form.html",
            "order": "NMFANTCLIE",
            "showDescriptionOrder": false,
            "actions": [
                {
                    "name": "CANCEL",
                    "label": "Cancelar",
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ClienteController.changeClienteActionEventCancel(args)"
                        }
                    ]
                },
                {
                    "name": "SAVE",
                    "label": "Trocar",
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ClienteController.changeClienteActionEventSave()"
                        }
                    ]
                }
            ],
            "events": [],
            "dataSource": {
                "name": "product#/currentCliente",
                "rest": true,
                "lazyLoad": true,
                "localStorage": false,
                "data": []
            },
            "fields": [
                {
                    "name": "NMFANTCLIE",
                    "label": "Cliente",
                    "isActive": true,
                    "focused": true,
                    "isVisible": true,
                    "hasPagination": false,
                    "searchWithPagination": false,
                    "template": "zh-select-autocomplete#field/select-autocomplete.html",
                    "searchableFields": ["NMFANTCLIE"],
                    "events": [
                        {
                            "name": "FieldOnChange",
                            "code": "ClienteController.changeClienteFieldOnChangeCliente(args)"
                        }
                    ],
                    "valueField": "CDCLIENTE",
                    "descriptionField": "NMFANTCLIE",
                    "descriptionFieldLabel": "Nome fantasia",
                    "showSelectAs": "grid",
                    "fields": [
                        {
                            "name": "NRORG",
                            "label": "NRORG",
                            "showOnList": true
                        },
                        {
                            "name": "NMFANTCLIE",
                            "label": "Nome Fantasia",
                            "showOnList": true
                        },
                        {
                            "name": "NMRAZSOCCLIE",
                            "label": "Razão Social",
                            "showOnList": true
                        }
                    ],
                    "outData": {
                        "NRINSJURCLIE": "NRINSJURCLIE",
                        "NMFANTCLIE": "NMFANTCLIE",
                        "NMRAZSOCCLIE": "NMRAZSOCCLIE",
                        "CDCLIENTE": "CDCLIENTE",
                        "NRORG": "NRORG",
                        "NRORGPRINCIPAL": "NRORGPRINCIPAL"
                    },
                    "dataSource": {
                        "name": "product#/findAvailableCliente",
                        "rest": true,
                        "localStorage": false,
                        "data": []
                    }
                },
                {
                    "name": "NRINSJURCLIE",
                    "label": "Número da inscrição jurídica",
                    "template": "field/text-edit.html",
                    "readOnly": true,
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true
                },
                {
                    "name": "NMFANTCLIE",
                    "label": "Nome Fantasia",
                    "template": "field/text-edit.html",
                    "readOnly": true,
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true
                },
                {
                    "name": "NMRAZSOCCLIE",
                    "label": "Razão Social",
                    "template": "field/text-edit.html",
                    "readOnly": true,
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true
                },
                {
                    "name": "NRORG",
                    "label": "Organização",
                    "template": "field/text-edit.html",
                    "readOnly": true,
                    "isVisible": false
                }
            ]
        };
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ClienteService', ClienteService);
});

function ClienteController(ScreenService, LibUtilities, restEngine, $location, templateManager, ClienteService, metaDataFactory, $rootScope, MessageService) {

    var self = this;

    var widgetChangeCliente = {};

    var getFieldClienteFromChangeCliente = function () {
        return widgetChangeCliente.getField('NMFANTCLIE');
    };

    var maskFormatFunction = function (value) {
        if (value) {
            var length = value.length;
            if (length === 11)
                value = value.replace(/^(\d{3})(\d{3})(\d{3})(\d{2})/, "$1.$2.$3-$4");
            else if (length === 14)
                value = value.replace(/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/, "$1.$2.$3/$4-$5");
        }
        return value;
    };

    this.changeCliente = function(){
        var widget = metaDataFactory.widgetFactory(ClienteService.getWidgetClienteJson(), templateManager.container);
        widgetChangeCliente = widget;
        widget.reload().then(function(){
                widgetChangeCliente.moveToFirst();
                widgetChangeCliente.currentRow.NRINSJURCLIE = maskFormatFunction(widgetChangeCliente.currentRow.NRINSJURCLIE);
            }
        );
        ScreenService.openPopup(widget);
    };

    this. openLogImportation = function() {
        ScreenService.openWindow("product#logImportation");
        ScreenService.toggleSideMenu();
    };

    this.changeClienteActionEventCancel = function(){
        ScreenService.closePopup();
    };

    this.changeClienteFieldOnChangeCliente = function(){
        widgetChangeCliente.currentRow.NRINSJURCLIE = maskFormatFunction(widgetChangeCliente.currentRow.NRINSJURCLIE);
    };

    var showErrorMessage = function(response) {
        MessageService.getMessageByCode('product#UNEXPECTED_ERROR').then(function(defaultMessage) {
            if (response && response.data && response.data.error) {
                var error = response.data.error;
                if (isJson(error)) {
                    error = JSON.parse(error);
                    var params = error.PARAMS || null;
                    var messageCode = error.CODE || 'UNEXPECTED_ERROR';
                    MessageService.getMessageByCode('product#' + messageCode).then(function(message) {
                        ScreenService.showMessage(message, 'ERROR', null, params);
                    }).catch(function() {
                        ScreenService.showMessage(messageCode, 'ERROR', null, params);
                    });
                } else {
                    var message = Util.isEmptyOrBlank(error) ? defaultMessage : error.split("Exception:").pop();
                    ScreenService.showMessage(message, 'ERROR');
                }
            } else if (response && response.message) {
                ScreenService.showMessage(response.message, 'ERROR');
            } else {
                ScreenService.showMessage(defaultMessage, 'ERROR');
            }
        });
    };

    var isJson = function(str) {
        var result = null;

        try {
            result = JSON.parse(str);
        } catch (e) { }

        return result;
    };

    var changeNrOrgCliente = function (){
        return restEngine.post({
            service: "product#/zmartNrOrgCliente",
            typeRequest: "POST",
            params: {
                "requestType": "FilterData",
                "filter": [
                    {
                        "name": "NRORG",
                        "operator": "=",
                        "value": widgetChangeCliente.currentRow.NRORG
                    },
                    {
                        "name": "NRORGPRINCIPAL",
                        "operator": "=",
                        "value": widgetChangeCliente.currentRow.NRORGPRINCIPAL
                    }
                ]
            },
            timeout: 120000
        });
    };
    var reloadALl = function () {
        window.location.reload();
    };

    var getIconFromType = function(type, defaultIcon) {
		var icon = defaultIcon;
		if (type) {
			switch (type.toUpperCase()) {
				case 'ERROR':
				case 'E':
					icon = 'error';
					break;
				case 'ALERT':
				case 'W':
					icon = 'exclamation';
					break;
				case 'INFORMATION':
					icon = 'information';
					break;
				case 'CONFIRMATION':
					icon = 'question';
					break;
				case 'SUCCESS':
				case 'M':
					icon = 'checked';
					break;
			}
		}
		return icon;
	};

    var confirmMessage = function(message, type, success, fail, icon, variables) {
		icon = icon ? icon : getIconFromType(type, 'question');
		return ScreenService.showCustomDialog(message, type, icon, [{
			label: ScreenService.i18n("Sim"),
			"default": true,
			code: function(e, deferred) {
				if (Util.isFunction(success)) {
					success(e);
				}
				deferred.resolve(e);
			}
		}, {
			label: ScreenService.i18n("Não"),
			code: function(e, deferred) {
				if (Util.isFunction(fail)) {
					fail(e);
				}
				deferred.reject(e);
			}
		}], variables);
	};

    this.changeClienteActionEventSave = function() {
        confirmMessage(ScreenService.i18n('Deseja realmente trocar o cliente?'), 'question', function () {
            changeNrOrgCliente().then(function(){
                setLocalVar("clienteNRORG",widgetChangeCliente.currentRow.NRORG);
                setLocalVar("clienteNRORGPRINCIPAL",widgetChangeCliente.currentRow.NRORGPRINCIPAL);
                LibUtilities.openWindow('product#mainDashboard');
                window.location.reload();
                Util.doAsync(function () {
                    reloadALl();
                },1000);
            });
        });

    }
}

Configuration(function (ContextRegister) {
    ContextRegister.register('ClienteController', ClienteController);
});
