module.exports = function(grunt) {

	var packageName = 'zh-crosstab';
	
	var servicesPath = 'src/services/**/*.js';
	var controllersPath = 'src/controllers/**/*.js';
	var directivesPath = 'src/directives/**/*.js';
	var registersPath = 'src/registers/**/*.js';
	var configPath = 'src/config/**/*.js';
	var assetsPath = ['src/**/*.js'];
	var exceptPath = ['!src/utilities/**/'];

	grunt.initConfig({
		jshint: {
			all: ['GruntFile.js', assetsPath, exceptPath]
		},
		watch: {
			pivotal: {
				files: [assetsPath],
				tasks: ['jshint']
			}
		},
		bump: {
		    options: {
		        files: ['bower.json', 'package.json'],
		        pushTo: 'origin',
		        commitFiles: ["-a"],
		        push: true
		    }
		},
		concat: {
			zh: {
				options: {
					process: function(src, filepath) {
						return '\n' + '// FILE: ' + filepath + '\n' + src;
					}
				},
				src: [configPath, servicesPath, controllersPath, directivesPath, registersPath, exceptPath],
				dest: 'dist/' + packageName + '.js'
			}
		},
		uglify: {
			options: {
				mangle: false
			},
			target: {
				files: {
					'dist/zh-crosstab.min.js': ['dist/' + packageName + '.js']
				}
			}
		},
		copy: {
			main: {
				expand: true,
				cwd: 'src/',
				src: 'templates/**/*',
				dest: 'dist/'
			}
		}
	});

	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-copy');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-bump');


	// Default task.
	grunt.registerTask('default', ['jshint', 'concat', 'uglify', 'copy']);
};