
// FILE: src/config/config.js
window.templateUrls = window.templateUrls || [];
var packageName = 'zh-select-autocomplete';

var hasPackageRegistered = window.templateUrls.some(function(templateUrl){
	return templateUrl.package === packageName;
});

if(!hasPackageRegistered){
	window.templateUrls.push({
		'package': packageName,
		'baseUrl': './bower_components/' + packageName + '/dist/templates/'
	});
}

// FILE: src/controllers/zh-select-autocomplete.controller.js
function ZhSelectAutoCompleteController($scope, $rootScope, $timeout, templateManager, ScreenService, ZhUtil, SelectService, ZHPromise, maskEngine, EventChildPropagation, DecodeValueFieldService) {

	var self = this;
	$scope.viewMore = 'View more...';

	/**
	 * Extends the behavior of it's parent (ListController).
	 */
	function extendBehavior() {
		Util.extend(self, new ListTemplatable());
	}
	extendBehavior();

	/**
	 * Build all properties of the field and, open it's popup.
	 *
	 * @function _openField
	 * @private
	 * @param   {Field} field - The field to has it's popup opened.
	 * @returns {Promise} A resolved promise.
	 */
	var _openField = function(field) {

		var defer = ZHPromise.defer();
		var actions = [{
			"label": "Cancel",
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"name": "ActionEvent",
				"code": "ScreenService.closePopup(); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
				"id": 9998
			}]
		}];

		if (field.showAddButtonAs == 'action') {
			actions.push({
				"icon": "plus",
				"showAsAction": "always|edit|checked_rows|view",
				"isVisible": true,
				"showOnForm": true,
				"showOnList": true,
				"events": [{
					"name": "ActionEvent",
					"code": "args.owner.widget.onAdd()",
					"id": 9999
				}]
			});
		}

		$scope.clearAutoCompleteListData();

		var template;
		if (!field.showSelectAs) {
			template = "widget/list.html";
		}
		else if (field.showSelectAs === "tree") {
			template = "widget/tree/grid.html";
		}
		else {
			template = "widget/" + field.showSelectAs + ".html";
		}

		var fields;

		function insertField(displayedField, key){
			field.fields[key] = {
				"name": displayedField,
				"showLabel": false,
				"showOnList": true,
				"mask": field.mask
			};
		}

		function insertLabels(label, key){
			field.fields[key].label = label;
		}

		if (field.fields && field.fields.length) {
			fields = field.fields;
		}
		else{
			field.fields = [];
			field.descriptionFields.forEach(insertField);
		}

		if(field.displayedFieldsLabels){
			field.displayedFieldsLabels.forEach(insertLabels);
		}

		var isToReload = !!(field.selectWidget && Util.objectHasAnyValue(field.selectWidget.searchCriteria));
		if (!field.selectWidget) {
			var widgetEvents = [{
				"name": "WidgetAfterMoveRow",
				"code": "args.owner.setSelected(args);",
				"id": 9999
			}];
			field.selectWidget = metaDataFactory.widgetFactory({
				"id": 0,
				"name": "Widget",
				"label": field.label,
				"template": template,
				"container": field.widget.container,
				"isVisible": true,
				"popupNoBlock": false,
				"showDescriptionOrder": false,
				"floatingControl": {
					"openSearch": !Util.isMobile()
				},

				"parentProperty": field.parentProperty,
				"primaryProperty": field.primaryProperty,
				"expandingProperty": field.expandingProperty,
				"breadcrumbProperty": field.breadcrumbProperty,
				"order": field.order,
				"reverse": field.reverse,
				"bringGroupsCollapsed": field.bringGroupsCollapsed,
				"orderFromGroups": field.orderFromGroups,
				"selectField": field,
				"showCheckbox": false,
				"onAdd": field.onAdd,
				"showAddButtonAs": field.showAddButtonAs,
				"addButtonText": field.addButtonText,
				"noMatchText": field.noMatchText,
				"searchNoMatchText": field.searchNoMatchText,
				"fields": field.fields,
				"dataSource": field.dataSource,
				"dataSourceFilter": field.dataSourceFilter,
				"itemsPerPage": field.itemsPerPage,
				"searchWithPagination": Util.isUndefinedOrNull(field.searchWithPagination) ? true : field.searchWithPagination,
				"hasPagination": Util.isUndefinedOrNull(field.hasPagination) ? true : field.hasPagination,
				"onDemand": field.onDemand,
				"allowSelectOnlyLastChild": field.allowSelectOnlyLastChild,
				"actions": actions,
				"events": EventChildPropagation.propagate(widgetEvents, field),
				"searchCriteria": field.searchCriteria
			});
			field.selectWidget.setSelected = setSelected;
			field.selectWidget.afterClosePopup = afterClosePopup;
			isToReload = !!(field.dataSource && field.dataSource.lazyLoad);
		}

		if (isToReload) {
			field.selectWidget.searchCriteria = {};
			field.selectWidget.searchList = "";
			field.reload().then(defer.resolve, defer.resolve);
		}
		else {
			defer.resolve();
		}

		ScreenService.openPopup(field.selectWidget);
		return defer.promise;
	};

	/**
	 * Config the properties valueFied and descriptionField because the controller works only with desctiptionFields and valueFields.
	 *
	 * @function configValueFieldsAndDescriptionFields
	 * @private
	 */
	function configValueFieldsAndDescriptionFields(){
		var field = $scope.field;
		if(!field.descriptionFields){
			field.descriptionFields = [field.descriptionField];
		}
		if(!field.valueFields){
			field.valueFields = [field.valueField];
		}
	}

	/**
	 * Open the popup of the given field.
	 *
	 * @function openField
	 * @param {Field} field - The field to be opened.
	 */
	$scope.openField = function(field) {
		if (!field.isReadOnly()) {
			if (field.beforeSelectOpen) {
				var e = field.beforeSelectOpen();
				if (e.isDefaultPrevented()) {
					return;
				}
			}
			// remove focus from element
			if (field.element) {
				field.element.blur();
				window.getSelection().removeAllRanges(); //bug webkit
			}

			_openField(field).then(function() {
				if (field.afterSelectOpen) {
					field.afterSelectOpen();
				}
			});
		}
	};

	/**
	 * Open the popup of the field on the scope.
	 */
	$scope.field.openField = function() {
		$scope.openField($scope.field);
	};

	/**
	 * Clear the outdata related to the component.
	 */
	$scope.field.clearByOutData = (function() {
		for (var prop in this.outData) {
			this.widget.currentRow[this.outData[prop]] = "";
		}
	}).bind($scope.field);

	/**
	 * Clear the value of the selected row by the component from the current row.
	 *
	 * @function clearByName
	 * @public
	 */
	$scope.field.clearByName = (function() {
		var currentRow = this.widget.currentRow;

		function clearCurrentRowValueFields(value){
			currentRow[value] = "";
		}

		currentRow[this.name] = "";
		this.valueFields.forEach(clearCurrentRowValueFields);
	}).bind($scope.field);

	if ($scope.selectField) {
		$scope.selectField.openField = function() {
			$scope.openField($scope.selectField);
		};
	}

	/**
	 * Set the widget of the component as changed.
	 *
	 * @function setChanged
	 * @private
	 * @param {Field} field - The field to have it's current row setted as changed.
	 */
	var setChanged = function(field) {
		field.widget.currentRow.changed = true;
	};

	/**
	 * Clear the filter of the component, and the values related to him from the current row.
	 *
	 * @function clearField
	 * @public
	 * @param {Field} field - The field to have it's values cleared.
	 */
	$scope.clearField = function(field) {
		if (field.isReadOnly()) {
			return;
		}
		field.clearByOutData();
		field.clearByName();
		field.dataSource.clearCheckedRows();
		setChanged(field);
		if (field.change) {
			field.change();
		}
		$scope.clearAutoCompleteListData();
		$scope.hideList();
		templateManager.updateTemplate();
	};

	$scope.showSpan = function() {
		var field = $scope.field;
		var currentRow = field.widget.currentRow;
		var hasValueSetted = !!currentRow[field.name];

		return !$scope.inputWithEditableContent.is(':focus') &&
						  hasValueSetted;
	};

	/**
	 * Searches for the given value if the value is valid.
	 *
	 * @function search
	 * @public
	 * @param {String} selectListSearch - The value to be searched. Should be bigger than 0.
	 * @param {Field} field - The field that will have the data that will be searched.
	 */
	$rootScope.search = function(selectListSearch, field) {
		var minLength = field.minimumInputLength || 0;
		if (selectListSearch.length > minLength) {
			innerSearch(selectListSearch, field);
		}
	};

	/**
	 * Change the dataSource's data to the filtered by the given value.
	 *
	 * @function innerSearch
	 * @private
	 * @param {String} selectListSearch - The value to be searched on the field.
	 * @param {Field} field - The field where will be searched the values and changed the data of the dataSource.
	 */
	function innerSearch(selectListSearch, field) {
		if (Util.isArray(field.filterDefinitions) && field.filterDefinitions.length) {
			var dataSource = field.dataSource;
			var filter = ZhUtil.buildFiltersInField(field, selectListSearch);
			dataSource.filter(filter, function(data) {
				dataSource.data = data;
				templateManager.updateTemplate();
			});
		}
	}

	/**
	 * Add the given item as selected
	 *
	 * @function addItemAsSelected
	 * @public
	 * @param {Object} item - The selected item.
	 * @param {Field} field - The item's field, where the item will be setted as selected.
	 */
	this.addItemAsSelected = function(item, field) {
		var parentWidget = field.widget;
		var outData = field.outData;

		(field.outData ? changeByOut : changeByName)(field, item);
		item.__isSelected = true;
		field.dataSource.checkedRows = [item];
		setChanged(field);

		if (field._afterSelected) {
			field._afterSelected(item);
		}
		if (field.change) field.change();
	};

	/**
	 * Returns the mounted description of the selected value.
	 *
	 * @function getDescriptionFromSelectedValue
	 * @public
	 * @returns {String} The value that should be displayed of the selected row.
	 */
	$scope.getDescriptionFromSelectedValue = function() {
		var fieldName = $scope.field.name;
		var currentRow = $scope.field.widget.currentRow;

		return currentRow[fieldName];
	};

	/**
	 * Returns the item's identifier.
	 *
	 * @function getItemIdentifier
	 * @public
	 * @param   {Object} item - The item to get it's identifier.
	 * @returns {String} The item's identifier.
	 */
	this.getItemIdentifier = function(item) {
		var identifier = [];
		var field = $scope.field;
		var valueFields = field.valueFields;

		for(var valueField in valueFields){
			identifier.push(item[valueFields[valueField]]);
		}

		return identifier.join(" | ");
	};

	/**
	 * Get the item with the given identifier.
	 *
	 * @function searchItem
	 * @private
	 * @param   {String} itemIdentifier - The desired item's identifier.
	 * @returns {Object} The item with the given identifier.
	 */
	function searchItem(itemIdentifier){
		var field = $scope.field;
		var data = field.dataSource.data;
		var valueFields = field.valueFields;
		var itemIdentifierArr = itemIdentifier.split(" | ");

		function findItem(item){
			var result = true;
			for(var valueField in valueFields){
				result &= item[valueFields[valueField]] == itemIdentifierArr[valueField];
			}
			return result;
		}

		item = data.find(findItem);
		return item;
	}

	/**
	 * Field's routine executed after one row be selected.
	 *
	 * @function onAutoCompleteItemSelect
	 * @public
	 * @param {String} selectedItemIdentifier - The identifier of the selected row.
	 */
	$scope.onAutoCompleteItemSelect = function(selectedItemIdentifier) {
		var item = searchItem(selectedItemIdentifier);

		self.clearEditableInput();
		self.addItemAsSelected(item, $scope.field);
		$scope.hideList();
	};

	/**
	 * Set the item passed as clicked, clearing the component's last input and hiding it's list.
	 *
	 * @function onAutoCompleteItemClick
	 * @public
	 * @param {Object} clickedItem - The clicked item, with it's data.
	 */
	this.onAutoCompleteItemClick = function(clickedItem) {
		self.clearEditableInput();
		self.addItemAsSelected(clickedItem, $scope.field);
		$scope.hideList();
	};

	/**
	 * Set the args' row as selected.
	 *
	 * @function setSelected
	 * @private
	 * @param {Object} args - The window args.
	 */
	function setSelected(args) {
		if (!$scope.field.allowSelectOnlyLastChild || ($scope.field.allowSelectOnlyLastChild && self.isTree($scope.field.selectWidget) && isLastChild(args.row))) {
			ScreenService.closePopup();
			self.onAutoCompleteItemClick(args.row);
		}
	}

	/**
	 * Tells if the given row is the last child.
	 *
	 * @function isLastChild
	 * @private
	 * @param   {Object} row - The row to be tested.
	 * @returns {Boolean} Result of the test of the row being the last child.
	 */
	var isLastChild = function(row) {
		return row._childCount === 0;
	};

	/**
	 * Routine executed after the field's popup be closed.
	 *
	 * @function afterClosePopup
	 * @private
	 * @param {Field} field - The field that had it's popup closed.
	 */
	function afterClosePopup(field) {
		if (field.element) {
			field.element.focus();
		}
	}

	/**
	 * Change the values on the current row used to identify the selected row and, print the fields value on the screen.
	 *
	 * @function changeByName
	 * @private
	 * @param {Field} field - The field to have it's values changed.
	 * @param {Object} row - The selected row.
	 */
	function changeByName(field, row) {
		var valueFields = field.valueFields;
		var descriptionFields = field.descriptionFields;
		var currentRow = field.widget.currentRow;
		currentRow[field.name] = "";


		function buildCurrentRowValueFields(value, key){
			currentRow[value] = row[value];
		}

		valueFields.forEach(buildCurrentRowValueFields);
		currentRow[field.name] = self.getItemDescription(row);
	}

	/**
	 * Change the given field current row value based on the configured outdata.
	 *
	 * @function changeByOut
	 * @private
	 * @param {Field} field - The field to have it's values changed.
	 * @param {Object} row - The selected row.
	 */
	function changeByOut(field, row) {
		if (field.widget.currentRow) {

			changeByName(field, row);

			var outData = field.outData || {};

			Object.keys(outData).forEach(function(key) {
				if (Array.isArray(outData[key])) {
					outData[key].forEach(function(valueKey) {
						field.widget.currentRow[valueKey] = row[key];
					});
				}
				else {
					field.widget.currentRow[outData[key]] = row[key];
				}
			});
		}
	}

	/**
	 * Get the component's actual value, on it's placeholder, returning the fieldRow, if it is passed.
	 *
	 * @function getSelectionText
	 * @public
	 * @param {Object} [fieldRow] - The clicked item, with it's data.
	 */
	this.getSelectionText = function getSelectionText(fieldRow) {
		if ($scope.field.placeholder && !fieldRow) {
			selectionText = $scope.field.placeholder;
		}
		else {
			selectionText = fieldRow;
		}
		return selectionText;
	};

	/**
	 * Open a new window.
	 *
	 * @param  {Field} field - The field with the container's name that will be opened.
	 */
	$scope.maintainWindow = function(field) {
		if (field.maintainWindow) {
			var maintainWindowFilter = SelectService.buildMaintainFilter(field);
			ScreenService.openWindow(field.maintainWindow, maintainWindowFilter);
		}
	};

	var lastSearch = "";
	$scope.localfilter = "";

	function updateDatasource(itemSearch){
		if($scope.field.dataSource.rest){
			if((itemSearch !== undefined) && itemSearch !== lastSearch){
				if($scope.field.searchableFields){
			        var fieldName = $scope.field.searchableFields.join("|");
			    	var filters = [
				    	{
			                "name": fieldName,
			                "operator": "LIKE_ALL",
			                "value": "%" + itemSearch + "%"
			            }
			    	];

		    		$scope.field.dataSource.load(filters, 1, 30);
		    		lastSearch = itemSearch;
		    		resetDropdown();
			    } else {
			    	throw new Error('Field ' + $scope.field.name + ' must have searchableFields. ');
			    }
			}
		} else {
			$scope.localfilter = itemSearch;
		}
    }

	$scope.$watchCollection('field.dataSourceFilter', propagateFieldDataSourceFilterToSelectWidget);
	$scope.$watch('itemSearch', Util.buildDebounceMethod(updateDatasource, 500, false));


	/**
	 * Propagate the field's DataSourceFilter to the selectWidget.
	 *
	 * @function propagateFieldDataSourceFilterToSelectWidget
	 * @private
	 * @param {DataSourceFilter} fieldDataSourceFilter - The field's DataSourceFilter to be propagated.
	 */
	function propagateFieldDataSourceFilterToSelectWidget(fieldDataSourceFilter) {
		if ($scope.field.selectWidget) {
			$scope.field.selectWidget.dataSourceFilter = fieldDataSourceFilter;
		}
	}

	var propertyToSearch = [];

	/**
	 * Initialize the field.
	 *
	 * @function initialize
	 */
	(function initialize() {
		initializeValuesOnCurrentRow();
		configValueFieldsAndDescriptionFields();
		buildPropertiesToSearch();
		return $scope.field.dataSource.data;
	})();

	/**
	 * Test the row to check if it should be displayed on the field's dropdown.
	 *
	 * @function searchFilter
	 * @public
	 * @param   {Object} row - The row to be tested.
	 * @returns {Boolean} Result tellling if the field should be displayed.
	 */
	$scope.searchFilter = function(row) {
		return Util.isUndefinedOrNull($scope.itemSearch) || propertyToSearch.some(function(data) {
			return ~row[data].indexOfLatin($scope.itemSearch);
		});
	};

	/**
	 * Clear the varibles related to the filter.
	 *
	 * @function clearSearchFilter
	 * @public
	 * @param   {Object} row - The row to be tested.
	 * @returns {Boolean} Result tellling if the field should be displayed.
	 */
	$scope.clearSearchFilter = function(){
		$scope.itemSearch = '';
	};

	/**
	 * Insert into the propertysToSearch the fields that the user can search on the field's dropdown.
	 *
	 * @function addFieldNamesToPropertyToSearch
	 * @private
	 * @param {Array.<Fields>} fields - The fields that should be avaible to search.
	 */
	function addFieldNamesToPropertyToSearch(fields) {
		fields.forEach(function(field){
			propertyToSearch.push(field);
		});
	}

	/**
	 * Build the properties that the user can search on the field's  dropdown.
	 *
	 * @function buildPropertiesToSearch
	 * @private
	 */
	function buildPropertiesToSearch(){
		var field = $scope.field;
		var propertyToAdd = null;
		if (!Util.isUndefined(field.searchableFields)) {
			propertyToAdd = (field.searchableFields);
		} else if (!Util.isUndefined(field.displayedFields)) {
			propertyToAdd = (field.displayedFields);
		} else {
			propertyToAdd = (field.descriptionFields);
		}

		addFieldNamesToPropertyToSearch(propertyToAdd);
	}

	/**
	 * Initilize the field.name value on it's currentRow.
	 *
	 * @function initializeValuesOnCurrentRow
	 * @private
	 */
	function initializeValuesOnCurrentRow() {
		var currentRow = $scope.field.widget.currentRow;
		var fieldName = $scope.field.name;
		currentRow[fieldName] = currentRow[fieldName] || "";
	}

	/**
	 * Remove the item from the current row by it's index.
	 *
	 * @function removeSelectedItemByIndex
	 * @private
	 */
	function removeSelectedItemByIndex(index) {
		var widgetParent = $scope.field.widget;
		var fieldName = $scope.field.name;
		widgetParent.currentRow[fieldName].splice(index, 1);
	}

	/**
	 * Functions used by template
	 **/

	/**
	 * Get the component's label.
	 *
	 * @function getFieldLabel
	 * @public
	 * @returns {String} The component's label.
	 */
	this.getFieldLabel = function() {
		return $scope.field.label;
	};

	/**
	 * Get the selected item's values.
	 *
	 * @function getSelectedvalues
	 * @public
	 * @returns {Object} The values of the selected item.
	 */
	this.getSelectedValues = function() {
		return $scope.field.widget.currentRow[$scope.field.name];
	};

	/**
	 * Get the selected item's description field.
	 *
	 * @function getSelectedItemDescription
	 * @public
	 * @returns {String} The component's data.
	 */
	this.getSelectedItemDescription = function(item) {
		return item[$scope.field.descriptionField];
	};

	/**
	 * Get the component's data.
	 *
	 * @function getData
	 * @public
	 * @returns {Array<Object>} The component's data.
	 */
	this.getData = function() {
		return $scope.field.dataSource.data;
	};

	$scope.dropDownLimit = 10;

	/**
	 * Updates the number of itens in the dropdown
	 *
	 * @function loadMore
	 * @public
	 */
	$scope.loadMore = function(){
		if($scope.dropDownLimit === 20){
			$scope.viewMore = "Open search widget";
			$scope.dropDownLimit += 10;
		} else if($scope.dropDownLimit >= 30){
			$scope.openField($scope.field);
			resetDropdown();
		} else {
			$scope.dropDownLimit += 10;
		}
	};

	function resetDropdown() {
		$scope.dropDownLimit = 10;
		$scope.viewMore = "View more...";
	}

	$scope.shouldShowLoadMore = function(){
		return $scope.field.dataSource.data.length >= $scope.dropDownLimit;
	};

	/**
	 * Traslate the text of the item according to field's option and i18n.
	 *
	 * @param  {Field}  field The field with the options.
	 * @param  {Object} item  The item With the text to be translated.
	 * @return {String}       The translated text.
	 */
	function translateText(field, item){
		if(!field.getTemplateSimpleName){
			Util.extend(field, new Templatable());
		}
		var text = item[field.name];
		return DecodeValueFieldService.model(field, text);
	}

	/**
	 * Returns the given item's text to be shown on the dropdown.
	 *
	 * @function getItemText
	 * @public
	 * @param   {Object} item - An item from the autocomplete field.
	 * @returns {String}        The item's description field if both have one, or an empty string.
	 */
	this.getItemText = function(item) {
		var itemDescription = [];
		var descriptionFields = $scope.field.descriptionFields;
		var displayedFields = $scope.field.displayedFields;
		var textSpacer = $scope.field.textSpacer || " | ";
		var fields = $scope.field.fields;

		function buildTranslatedItemDescription(description, key){
			var field = fields.filter(function(field){
				return field.name === description;
			})[0];
			itemDescription[key] = translateText(field, item);
		}

		function buildItemDescription(description, key){
			itemDescription[key] = item[description];
		}

		if(displayedFields){
			if(fields){
				displayedFields.forEach(buildTranslatedItemDescription);
			}
			else {
				displayedFields.forEach(buildItemDescription);
			}
		}
		else {
			if(fields){
				descriptionFields.forEach(buildTranslatedItemDescription);
			}
			else {
				descriptionFields.forEach(buildItemDescription);
			}
		}
		return itemDescription.join(textSpacer);
	};


	/**
	 * Returns the given item's descriptionField if the field has an descriptionField.
	 *
	 * @function getItemDescription
	 * @public
	 * @param   {Object} item - An item from the autocomplete field.
	 * @returns {String}        The item's description field if both have one, or an empty string.
	 */
	this.getItemDescription = function(item){
		var itemDescription = [];
		var descriptionFields = $scope.field.descriptionFields;
		var textSpacer = $scope.field.textSpacer || " | ";
		var fields = $scope.field.fields;

		function buildTranslatedItemDescription(description, key){
			var field = fields.filter(function(field){
				return field.name === description;
			})[0];
			itemDescription[key] = translateText(field, item);
		}

		function buildItemDescription(description, key){
			itemDescription[key] = item[description];
		}

		if(fields){
			descriptionFields.forEach(buildTranslatedItemDescription);
		}
		else {
			descriptionFields.forEach(buildItemDescription);
		}
		itemDescription = itemDescription.join(textSpacer);

		return itemDescription;
	};

	/**
	 * Remove the item with the given index from the parent field's current row.
	 *
	 * @function onRemoveSelectedItem
	 * @public
	 * @param {Integer} item - The index of the item to be removed.
	 */
	this.onRemoveSelectedItem = function(removedItemIndex) {
		removeSelectedItemByIndex(removedItemIndex);
	};

	/**
	 * Functions called from directive
	 *
	 **/

	/**
	 * Filter the data source by the text written.
	 *
	 * @function filterDataSource
	 */
	$scope.filterDataSource = function() {
		$scope.openSearchListAfterStartingWriting();
	};

	/**
	 * Functions used internally
	 **/

	/**
	 * Clear the field's search parameter and it's list data.
	 *
	 * @function clearAutoCompleteListData
	 */
	$scope.clearAutoCompleteListData = function() {
		$scope.itemSearch = '';
		$scope.clearInputFieldText();
		$scope.autoCompleteListData = [];
	};

	/**
	 * Clear the value on the field's input.
	 *
	 * @function clearEditableInput
	 * @public
	 */
	this.clearEditableInput = function() {
		$scope.inputWithEditableContent.val('');
	};

	/**
	 * Tells if the component should display it's search icon.
	 *
	 * @function showSearchIcon
	 * @public
	 * @param   {Object} fieldRow - The field row to be compared.
	 * @returns {Boolean} Value telling if the field's search icon should be displayed.
	 */
	this.showSearchIcon = function() {
		return !$scope.showSpan();
	};

	/**
	 * Tells if the component should display it's 'x' icon.
	 *
	 * @function showCloseXIcon
	 * @public
	 * @param   {Object} fieldRow - The field row to be compared.
	 * @returns {Boolean} Value telling if the field's 'x' icon should be displayed.
	 */
	this.showCloseXIcon = function() {
		return $scope.showSpan();
	};
}

// FILE: src/directives/zh-select-autocomplete.directive.js
/**
 *
 */
(function() {

    var ZeedhiDirectives = angular.module('ZeedhiDirectives');
    ZeedhiDirectives.directive('zhSelectAutocomplete', ['$timeout', zhSelectAutocomplete]);

    function zhSelectAutocomplete($timeout, ZhUtil) {

        var EDITABLE_INPUT_MIN_WIDTH = 130;
        var ENTER = 13;
        var SPACE = 32;
        var DOWN = 40;
        var ESC = 27;
        var TAB = 9;
        var UP = 38;

        var linkFunction = function(theScope, theElement, theAttributes, theController) {

            var scope = theScope;
            var element = theElement;
            var descriptionField = scope.field.descriptionField;
            var attributes = theAttributes;
            var controller = theController;
            var debounceTimeout;
            var selectedList = '';
            var selectedItem = '';
            var displayedFields = scope.field.displayedFields;

            scope.selectAutoCompleteElement = $(element);
            scope.inputWithEditableContent = $(element).find('.editable');
            scope.spanValueField = $(element).find('span');

            createKeyPressListener();
            createFocusListener();
            clickListener();
            KeyPressNavegationListener();

            /**
             * Creates the click clickListener.
             *
             * @function clickListener
             * @private
             */
            function clickListener() {
                var selectedClass = $(element).find('.selected-autocomplete');
                var showSelect = $(element).find(".show-select");
                selectedClass.click(function(event) {
                    if(!scope.itemSearch){
                        scope.itemSearch = "";
                    }
                    scope.openSearchListAfterStartingWriting();
                });
                showSelect.click(function(event) {
                    event.stopPropagation();
                });
            }

            /**
             * Creates the focus listener.
             *
             * @function createFocusListener
             * @private
             */
            function createFocusListener() {
                scope.inputHasFocus = false;
                scope.inputWithEditableContent.focus(function(event) {
                    scope.inputHasFocus = true;
                    scope.inputWithEditableContent.val(scope.itemSearch);
                    scope.inputWithEditableContent.select();
                    scope.inputWithEditableContent.parent().scrollTop(0);
                    scope.openSearchListAfterStartingWriting();
                    scope.$apply();
                });

                scope.inputWithEditableContent.blur(function(event) {
                    scope.inputHasFocus = false;
                    scope.$apply();
                });
            }

            /**
             * Enable the field's input.
             *
             * @function enableInputField
             * @private
             */
            function enableInputField() {
                scope.selectAutoCompleteElement.find("input:text").prop('disabled', false);
            }

            // register jQuery extension to select by TAB and Navation by press TAB
            jQuery.extend(jQuery.expr[':'], {
                focusable: function (el, index, selector) {
                    return $(el).is('a, button, :input, [tabindex]');
                }
            });

            /**
             * Creates the key press navigation listener.
             *
             * @function KeyPressNavegationListener
             * @private
             */
            function KeyPressNavegationListener() {
                var next;
                var navegationKeyPress = scope.selectAutoCompleteElement;
                navegationKeyPress.on('keydown', 'input,select', function(event) {
                    var EVENTKEY = event;
                    var spanList = scope.selectAutoCompleteElement.find('.show-select span');

                    //Enter, Space or Tab
                    if (EVENTKEY.which === ENTER || EVENTKEY.which === TAB) {
                        EVENTKEY.preventDefault();

                        if(canSelect(selectedList)){
                            scope.selectItem(selectedList);
                        }

                        if(getFocusableItems().length > 1){
                            scope.focusOnNextItem();
                            scope.hideList();
                            scope.inputWithEditableContent.val("");
                        }
                    }
                    //Down
                    else if (EVENTKEY.which === DOWN) {
                        if (selectedList) {
                            selectedList.removeClass('selected');
                            next = selectedList.next();
                            if (next.length > 0) {
                                selectedList = next.addClass('selected');
                            }
                            else {
                                selectedList = spanList.eq(0).addClass('selected');
                            }
                        }
                        else {
                            selectedList = spanList.eq(0).addClass('selected');
                        }
                    }
                    //Up
                    else if (EVENTKEY.which === UP) {
                        if (selectedList) {
                            selectedList.removeClass('selected');
                            next = selectedList.prev();
                            if (next.length > 0) {
                                selectedList = next.addClass('selected');
                            }
                            else {
                                selectedList = spanList.last().addClass('selected');
                            }
                        }
                        else {
                            selectedList = spanList.last().addClass('selected');
                        }
                    }
                    //Esc
                    else if (EVENTKEY.which === ESC) {
                        scope.hideList();
                        selectedList = '';
                        return false;
                    }
                    if (navegationKeyPress) {
                        var itemList = scope.selectAutoCompleteElement.find('.list-items');
                        var item = navegationKeyPress.find('.selected');
                        if (item.offset() !== undefined) {
                            var index = item.attr('data-item-index');
                            var movimentation = 0;
                            if(item.position().top + item.outerHeight() > itemList.innerHeight()){
                                movimentation = (index * 32 + item.outerHeight() - itemList.innerHeight());
                                animateItemFocus(itemList, movimentation);
                            } else if(item.position().top < 0){
                                movimentation = index * 32;
                                animateItemFocus(itemList, movimentation);
                            }
                        }
                    }

                });
            }

            /**
             * Makes the animation to scroll the item for the one who has focus.
             *
             * @function animateItemFocus
             * @private
             * @param  {Object} itemList      - The element witch has the list of items
             * @param  {Number} movimentation - The value in pixels of the scrolling distance from the top.
             */
            function animateItemFocus(itemList, movimentation){
                itemList.finish().animate({
                    scrollTop: movimentation
                }, 500);
            }

            /**
             * Get all the focusabel items.
             *
             * @function getFocusableItems
             * @private
             * @returns {Array.<Object>} The focusable items on the screen.
             */
            function getFocusableItems(){
                return $(':focusable');
            }

            /**
             * Test if the given selectedList is a valid row to be selected.
             *
             * @function canSelect
             * @private
             * @returns {Boolean} Result telling if the selectedList is a valid row to be selected.
             */
            function canSelect(selectedList){
                return (typeof selectedList === "string" && selectedList !== "") ||
                        (typeof selectedList === "object" && selectedList.text() !== "");
            }

            /**
             * Creates the key press listener.
             *
             * @function createKeyPressListener
             * @private
             */
            function createKeyPressListener() {
                scope.inputWithEditableContent.keyup(function(event) {
                    if (event.which !== SPACE && event.which !== ESC && event.which !== DOWN && event.which !== UP && event.which !== TAB) {
                        deleteOldDebounceTimeout();
                        createNewDebounceTimeout();
                    }
                });
            }

            /**
             * Deletes the actual debounce timeout.
             *
             * @function deleteOldDebounceTimeOut
             * @private
             */
            function deleteOldDebounceTimeout() {
                $timeout.cancel(debounceTimeout);
            }

            /**
             * Creates a debounce timeout to keep updating the dataSource.
             *
             * @function createNewDebounceTimeout
             * @private
             */
            function createNewDebounceTimeout() {
                debounceTimeout = $timeout(function() {
                    scope.filterDataSource();
                }, 500);
            }

            /**
             * Open the field's dropdown list.
             *
             * @function openSearchListAfterStartingWriting
             */
            scope.openSearchListAfterStartingWriting = function() {
                $(".zh-select-autocomplete .show-select").css("display", "none");
                scope.selectAutoCompleteElement.find(".show-select").css("display", "block");
                scope.selectAutoCompleteElement.find('.list-items').scrollTop(0);
            };

            /**
             * Clear the field input's text.
             *
             * @function clearInputFieldText
             */
            scope.clearInputFieldText = function() {
                scope.selectAutoCompleteElement.find('input:text').val('');
            };

            /**
             * Hide the fields's dropdown list.
             *
             * @function hideList
             */
            scope.hideList = function() {
                scope.selectAutoCompleteElement.find(".show-select").hide();
                scope.selectAutoCompleteElement.find('.show-select span').removeClass('selected');
            };

            /**
             * Set as selected the given item.
             *
             * @function selectItem
             * @param {Object} selectedList - The DOM element of the selected row.
             */
            scope.selectItem = function(selectedList){
                if(!selectedList) return;
                selectedItem = selectedList.text();
                var itemIdentifier = selectedList.attr("data-item-identifier");
                scope.onAutoCompleteItemSelect(itemIdentifier);
                scope.$apply();
            };

            /**
             * Focus on the next item of the screen.
             *
             * @function focusOnNextItem
             */
            scope.focusOnNextItem = function(){
                scope.inputHasFocus = false;
                $canfocus = getFocusableItems();
                var index = $canfocus.index(document.activeElement) + 1;
                if (index >= $canfocus.length) index = 0;
                $canfocus.eq(index).focus();
                scope.inputWithEditableContent.blur();
            };

            scope.focusOnInput = function(){
                $timeout(function(){scope.inputWithEditableContent.focus();});
            };

            /**
             * Treats the click outside the component.
             */
            $(document).on("click", function(event) {
                scope.selectAutoCompleteElement[0].focus();
                if (!$.contains(scope.selectAutoCompleteElement[0], event.target)) {
                    scope.hideList();
                    scope.inputWithEditableContent.val("");
                }
            });
        };
        return {
            restrict: 'A',
            link: linkFunction
        };
    }
})();