/**
 *
 */
(function() {

    var ZeedhiDirectives = angular.module('ZeedhiDirectives');
    ZeedhiDirectives.directive('zhSelectAutocomplete', ['$timeout', zhSelectAutocomplete]);

    function zhSelectAutocomplete($timeout, ZhUtil) {

        var EDITABLE_INPUT_MIN_WIDTH = 130;
        var ENTER = 13;
        var SPACE = 32;
        var DOWN = 40;
        var ESC = 27;
        var TAB = 9;
        var UP = 38;

        var linkFunction = function(theScope, theElement, theAttributes, theController) {

            var scope = theScope;
            var element = theElement;
            var descriptionField = scope.field.descriptionField;
            var attributes = theAttributes;
            var controller = theController;
            var debounceTimeout;
            var selectedList = '';
            var selectedItem = '';
            var displayedFields = scope.field.displayedFields;

            scope.selectAutoCompleteElement = $(element);
            scope.inputWithEditableContent = $(element).find('.editable');
            scope.spanValueField = $(element).find('span');

            createKeyPressListener();
            createFocusListener();
            clickListener();
            KeyPressNavegationListener();

            /**
             * Creates the click clickListener.
             *
             * @function clickListener
             * @private
             */
            function clickListener() {
                var selectedClass = $(element).find('.selected-autocomplete');
                var showSelect = $(element).find(".show-select");
                selectedClass.click(function(event) {
                    if(!scope.itemSearch){
                        scope.itemSearch = "";
                    }
                    scope.openSearchListAfterStartingWriting();
                });
                showSelect.click(function(event) {
                    event.stopPropagation();
                });
            }

            /**
             * Creates the focus listener.
             *
             * @function createFocusListener
             * @private
             */
            function createFocusListener() {
                scope.inputHasFocus = false;
                scope.inputWithEditableContent.focus(function(event) {
                    scope.inputHasFocus = true;
                    scope.inputWithEditableContent.val(scope.itemSearch);
                    scope.inputWithEditableContent.select();
                    scope.inputWithEditableContent.parent().scrollTop(0);
                    scope.openSearchListAfterStartingWriting();
                    scope.$apply();
                });

                scope.inputWithEditableContent.blur(function(event) {
                    scope.inputHasFocus = false;
                    scope.$apply();
                });
            }

            /**
             * Enable the field's input.
             *
             * @function enableInputField
             * @private
             */
            function enableInputField() {
                scope.selectAutoCompleteElement.find("input:text").prop('disabled', false);
            }

            // register jQuery extension to select by TAB and Navation by press TAB
            jQuery.extend(jQuery.expr[':'], {
                focusable: function (el, index, selector) {
                    return $(el).is('a, button, :input, [tabindex]');
                }
            });

            /**
             * Creates the key press navigation listener.
             *
             * @function KeyPressNavegationListener
             * @private
             */
            function KeyPressNavegationListener() {
                var next;
                var navegationKeyPress = scope.selectAutoCompleteElement;
                navegationKeyPress.on('keydown', 'input,select', function(event) {
                    var EVENTKEY = event;
                    var spanList = scope.selectAutoCompleteElement.find('.show-select span');

                    //Enter, Space or Tab
                    if (EVENTKEY.which === ENTER || EVENTKEY.which === TAB) {
                        EVENTKEY.preventDefault();

                        if(canSelect(selectedList)){
                            scope.selectItem(selectedList);
                        }

                        if(getFocusableItems().length > 1){
                            scope.focusOnNextItem();
                            scope.hideList();
                            scope.inputWithEditableContent.val("");
                        }
                    }
                    //Down
                    else if (EVENTKEY.which === DOWN) {
                        if (selectedList) {
                            selectedList.removeClass('selected');
                            next = selectedList.next();
                            if (next.length > 0) {
                                selectedList = next.addClass('selected');
                            }
                            else {
                                selectedList = spanList.eq(0).addClass('selected');
                            }
                        }
                        else {
                            selectedList = spanList.eq(0).addClass('selected');
                        }
                    }
                    //Up
                    else if (EVENTKEY.which === UP) {
                        if (selectedList) {
                            selectedList.removeClass('selected');
                            next = selectedList.prev();
                            if (next.length > 0) {
                                selectedList = next.addClass('selected');
                            }
                            else {
                                selectedList = spanList.last().addClass('selected');
                            }
                        }
                        else {
                            selectedList = spanList.last().addClass('selected');
                        }
                    }
                    //Esc
                    else if (EVENTKEY.which === ESC) {
                        scope.hideList();
                        selectedList = '';
                        return false;
                    }
                    if (navegationKeyPress) {
                        var itemList = scope.selectAutoCompleteElement.find('.list-items');
                        var item = navegationKeyPress.find('.selected');
                        if (item.offset() !== undefined) {
                            var index = item.attr('data-item-index');
                            var movimentation = 0;
                            if(item.position().top + item.outerHeight() > itemList.innerHeight()){
                                movimentation = (index * 32 + item.outerHeight() - itemList.innerHeight());
                                animateItemFocus(itemList, movimentation);
                            } else if(item.position().top < 0){
                                movimentation = index * 32;
                                animateItemFocus(itemList, movimentation);
                            }
                        }
                    }

                });
            }

            /**
             * Makes the animation to scroll the item for the one who has focus.
             *
             * @function animateItemFocus
             * @private
             * @param  {Object} itemList      - The element witch has the list of items
             * @param  {Number} movimentation - The value in pixels of the scrolling distance from the top.
             */
            function animateItemFocus(itemList, movimentation){
                itemList.finish().animate({
                    scrollTop: movimentation
                }, 500);
            }

            /**
             * Get all the focusabel items.
             *
             * @function getFocusableItems
             * @private
             * @returns {Array.<Object>} The focusable items on the screen.
             */
            function getFocusableItems(){
                return $(':focusable');
            }

            /**
             * Test if the given selectedList is a valid row to be selected.
             *
             * @function canSelect
             * @private
             * @returns {Boolean} Result telling if the selectedList is a valid row to be selected.
             */
            function canSelect(selectedList){
                return (typeof selectedList === "string" && selectedList !== "") ||
                        (typeof selectedList === "object" && selectedList.text() !== "");
            }

            /**
             * Creates the key press listener.
             *
             * @function createKeyPressListener
             * @private
             */
            function createKeyPressListener() {
                scope.inputWithEditableContent.keyup(function(event) {
                    if (event.which !== SPACE && event.which !== ESC && event.which !== DOWN && event.which !== UP && event.which !== TAB) {
                        deleteOldDebounceTimeout();
                        createNewDebounceTimeout();
                    }
                });
            }

            /**
             * Deletes the actual debounce timeout.
             *
             * @function deleteOldDebounceTimeOut
             * @private
             */
            function deleteOldDebounceTimeout() {
                $timeout.cancel(debounceTimeout);
            }

            /**
             * Creates a debounce timeout to keep updating the dataSource.
             *
             * @function createNewDebounceTimeout
             * @private
             */
            function createNewDebounceTimeout() {
                debounceTimeout = $timeout(function() {
                    scope.filterDataSource();
                }, 500);
            }

            /**
             * Open the field's dropdown list.
             *
             * @function openSearchListAfterStartingWriting
             */
            scope.openSearchListAfterStartingWriting = function() {
                $(".zh-select-autocomplete .show-select").css("display", "none");
                scope.selectAutoCompleteElement.find(".show-select").css("display", "block");
                scope.selectAutoCompleteElement.find('.list-items').scrollTop(0);
            };

            /**
             * Clear the field input's text.
             *
             * @function clearInputFieldText
             */
            scope.clearInputFieldText = function() {
                scope.selectAutoCompleteElement.find('input:text').val('');
            };

            /**
             * Hide the fields's dropdown list.
             *
             * @function hideList
             */
            scope.hideList = function() {
                scope.selectAutoCompleteElement.find(".show-select").hide();
                scope.selectAutoCompleteElement.find('.show-select span').removeClass('selected');
            };

            /**
             * Set as selected the given item.
             *
             * @function selectItem
             * @param {Object} selectedList - The DOM element of the selected row.
             */
            scope.selectItem = function(selectedList){
                if(!selectedList) return;
                selectedItem = selectedList.text();
                var itemIdentifier = selectedList.attr("data-item-identifier");
                scope.onAutoCompleteItemSelect(itemIdentifier);
                scope.$apply();
            };

            /**
             * Focus on the next item of the screen.
             *
             * @function focusOnNextItem
             */
            scope.focusOnNextItem = function(){
                scope.inputHasFocus = false;
                $canfocus = getFocusableItems();
                var index = $canfocus.index(document.activeElement) + 1;
                if (index >= $canfocus.length) index = 0;
                $canfocus.eq(index).focus();
                scope.inputWithEditableContent.blur();
            };

            scope.focusOnInput = function(){
                $timeout(function(){scope.inputWithEditableContent.focus();});
            };

            /**
             * Treats the click outside the component.
             */
            $(document).on("click", function(event) {
                scope.selectAutoCompleteElement[0].focus();
                if (!$.contains(scope.selectAutoCompleteElement[0], event.target)) {
                    scope.hideList();
                    scope.inputWithEditableContent.val("");
                }
            });
        };
        return {
            restrict: 'A',
            link: linkFunction
        };
    }
})();