function DashboardComparePeriodsController($scope, $rootScope, $timeout, ScreenService, templateManager, ZHPromise, eventEngine, Query,
                                  CrosstabService, $filter, maskEngine, ZhUtil, $window, ApplicationContext) {
    // BEGIN: Initialize
    var self = this;
    self.periods = {};
    var moment = $window.moment;
    var allowMultiSelect = null;
    $scope.filtered = [];
    self.orderByCustomListAsc = [];
    self.orderByCustomListDesc = [];

    this.__defineGetter__('widget', function() {
        return $scope.widget || $scope._currentPopup;
    });
    // END: Initialize

    // BEGIN: Utilities
    var CROSSTAB_TEMPLATE = 'zh-crosstab#widget/crosstab.html';
    var TREE_TEMPLATE = 'tree/grid.html';
    var TREE_LIST_TEMPLATE = 'tree/list.html';
    var GANTT_TEMPLATE = 'tree/gantt.html';

    var _isTemplate = function(template) {
        return self.widget.template && self.widget.template.indexOf(template) !== -1;
    };

    this.isTree = function() {
        return _isTemplate(TREE_TEMPLATE) || _isTemplate(TREE_LIST_TEMPLATE);
    };

    this.isGantt = function() {
        return _isTemplate(GANTT_TEMPLATE);
    };

    this.isCrosstab = function() {
        return _isTemplate(CROSSTAB_TEMPLATE);
    };
    // END: Utilities

    // BEGIN: Search
    $scope.searchList = this.widget.searchList;

    this.search = function(item) {
        self.widget.searchList = $scope.searchList;
        item.__notSelectable = false;
        if ($scope.searchList) {
            var returnValue = false;
            $.each(item, function(key, value) {
                var field = $.grep(self.widget.fields, function(field) {
                    return field.name === key;
                })[0];
                if(field && field.translate){
                    value = ScreenService.i18n(value);
                }
                returnValue = (!Util.isUndefinedOrNull(field) && field.isVisible && field.showOnList && !field.skipSearch &&
                    !Util.isUndefinedOrNull(value) && value.toString().indexOfLatin($scope.searchList) !== -1);
                return !returnValue;
            });
            item.__notSelectable = !returnValue;
            return returnValue;
        } else {
            return true;
        }
    };

    function getLikeAllFilter(widget){
        var filter = [];

        if (!Util.isEmptyOrBlank($scope.searchList)) {
            var columns = widget.fields
                .filter(function(field){
                    return field.isVisible && field.aclIsVisible && field.showOnList && !field.skipSearch;
                })
                .map(function(field){
                    return field.name;
                })
                .join('|');
            filter = Query.build().where(columns).likeAll("%" + $scope.searchList + "%").where();
        }

        return filter;
    }

    var _searchWithPaginationReload = Util.buildDebounceMethod(function(widget, callback) {
        var deferred = ZHPromise.defer();
        if(Util.isArray(self.widget.fields) && self.widget.fields.length) {
            widget.reload(getLikeAllFilter(self.widget)).then(function(data) {
                if (callback) callback();
                deferred.resolve(data);
            }, function() {
                deferred.reject();
            });
        } else {
            deferred.reject();
        }
        return deferred;
    }, 1000);

    var _resizeGrid = Util.buildDebounceMethod(function() {
        self.widget.resizeGridColumns();
    }, 1000);

    if (!self.isGantt() && !self.isCrosstab()) {
        $scope.$watch('searchList', function(current, old) {
            if (self.widget.searchWithPagination && !Util.isUndefined(current)) {
                _searchWithPaginationReload(self.widget, function() {
                    updateGroupedData();
                });
            }
            if (!Util.isUndefined(current) && current != old && self.widget.resizeGridColumns) { // new grid
                _resizeGrid();
            }
        });
    }
    // END: Search

    // BEGIN: Toggle
    function deselectAll(widget) {
        var event = widget.unSelectAll();
        if (!event || !event.isDefaultPrevented()) {
            widget.dataSource.clearCheckedRows();
            if (!$scope._currentPopup) {
                $rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
            }
        }
    }

    function selectAll(widget) {
        widget.dataSource.data.filter(self.search).forEach(function(row) {
            row.__isSelected = true;
            widget.dataSource.addCheckedRows(row);
        });
        if (!$scope._currentPopup) {
            $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
        }
    }

    $rootScope.$on("ui.deselectAll", function(event, widget) {
        deselectAll(widget);
    });

    this.onLongTap = function(row) {
        if (row.__group) return;

        if (this.widget.allowSelectOnLongTap) {
            this.toggle(row);
        }
    };

    this.toggle = function(row) {
        var widget = self.widget;
        widget.selectedRow = row;

        var event;
        if (widget.beforeSelectRow && !row.__isSelected) {
            event = widget.beforeSelectRow();
        } else if (widget.beforeUnselectRow && row.__isSelected) {
            event = widget.beforeUnselectRow();
        }

        if (!event || !event.isDefaultPrevented()) {

            var index = getIndex(row, $scope);
            if (index === -1) {
                row.__isSelected = true;
                if (!$scope._currentPopup) $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
            } else {
                delete row.__isSelected;
                widget.dataSource.removeCheckedRows(row);
            }

            if (self.isTree()) {
                self.handleDependentRows(row, widget);
            }

            if (!$scope._currentPopup && !widget.dataSource.checkedRows.length) {
                $rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
            }

            widget.updateCheckedRows();

            if (widget.afterSelectRow && row.__isSelected) {
                widget.afterSelectRow();
            } else if (widget.afterUnselectRow && !row.__isSelected) {
                widget.afterUnselectRow();
            }

        }
    };

    $scope.widget.toggle = self.toggle;

    this.toggleAll = function() {
        if (this.widget.hasCheckedRows()) {
            this.widget.unSelectAll();
        } else {
            this.widget.selectAll();
        }
        this.widget.updateCheckedRows();
    };

    this.widget.toggle = this.toggle;
    // END: Toggle

    // BEGIN: Others functions
    $scope.openRow = function(row, widget) {
        if (row.__group) return;

        if (widget.dataSource.getCheckedRows().length || allowMultiSelect === widget || widget.allowMultiSelect) {
            self.toggle(row);
        } else {
            widget.setCurrentRow(row);
        }
        widget.dataSource.updateCheckedRows();
    };

    $scope.openRowInnerList = function(row, widget) {
        if (typeof(widget.currentRow) !== "undefined" &&
            widget.currentRow.__index === widget.dataSource.data.indexOf(row)) {
            delete widget.currentRow;
        } else {
            $scope.openRow(row, widget);
        }
    };

    function getIndex(row, scope) {
        return self.widget.dataSource.checkedRows.indexOf(row);
    }

    $scope.scrollTo = function(evt) {
        $timeout(function() {
            var item = $(evt.target).closest('li');
            var itemTop = item.position().top;
            var itemBottom = itemTop + item[0].clientHeight;
            var scroller = item.closest("[zh-scroller]");
            var top = 0;
            if (itemTop < 0 || itemBottom > scroller.height()) {
                if (itemTop < 0) {
                    top = scroller.scrollTop() + itemTop - 10;
                } else {
                    top = scroller.scrollTop() + itemBottom - scroller.height() + 10;
                }
                scroller.animate({
                    scrollTop: top
                }, 500);
            }
        }, 100, true);
    };

    $rootScope.$on("ui.allowMultiSelect", function(event, widget) {
        if ($scope._currentPopup) {
            return;
        }
        allowMultiSelect = widget;
    });

    $rootScope.$on("ui.disableMultiSelect", function() {
        if ($scope._currentPopup) {
            return;
        }
        allowMultiSelect = null;
    });
    // END: Others functions

    // BEGIN: Conditional actions control

    var hasConditionalActions = false;
    var leftActions = [];
    var rightActionsGrid = [];
    var rightActionsList = [];
    var rightActions = [];
    var defaultActionsValue = function(){
        $scope.visibleLeftActions = leftActions.length;
        $scope.visibleRightActions = rightActions.length;
        $scope.leftActions = leftActions;
        $scope.rightActions = rightActions;
    };
    var buildGridActions = function(){
        hasConditionalActions = false;
        leftActions = $scope.widget.getActionsByVisibility('grid-left');
        rightActionsGrid = $scope.widget.getActionsByVisibility('grid-right');
        rightActionsList = $scope.widget.getActionsByVisibility('list-right');
        rightActions = rightActionsGrid.concat(rightActionsList);

        $scope.widget.actions.forEach(function(item) {
            if (item.condition) {
                hasConditionalActions = true;
            }
        });

        if(!hasConditionalActions){
            defaultActionsValue();
        }
    };
    $scope.buildGridActions = buildGridActions;
    buildGridActions();
    //wands
    $scope.$watch("widget.dataSource.data", function(data) {

        if(hasConditionalActions){

            $scope.visibleLeftActions = 0;
            $scope.visibleRightActions = 0;

            if ($scope.widget && $scope.widget.dataSource && $scope.widget.dataSource.data) {
                $scope.widget.dataSource.data.forEach(function(row) {
                    ApplicationContext.registerInstance('row', row || {});
                    row.__rightActions = rightActions.filter(function(action) {
                        return !action.condition || ZhUtil.evaluate("@"+action.condition+"@");
                    });

                    row.__leftActions = leftActions.filter(function(action) {
                        return !action.condition || ZhUtil.evaluate("@"+action.condition+"@");
                    });
                    $scope.visibleRightActions = Math.max($scope.visibleRightActions, row.__rightActions.length);
                    $scope.visibleLeftActions = Math.max($scope.visibleLeftActions, row.__leftActions.length);
                    ApplicationContext.removeInstance('row');
                });
            }
        }
    });


    // END: Conditional actions control

    // BEGIN: Order by
    //wands
    this.changeOrder = function(column) {
        if (!!column.disableOrdering) return;

        if (self.widget.order == column.name && self.widget.reverse) {
            self.widget.order = undefined;
            self.widget.reverse = false;
        } else {
            self.widget.reverse = (self.widget.order == column.name) ? !self.widget.reverse : false;
            self.widget.order = column.name;
        }
        updateGroupedData();
    };

    var orderByColumns;
    var orderByFieldFlow = {
        "datetime.html": getDatetimeProxyToOrderBy,
        "calendar.html": getDateProxyToOrderBy,
        "select-month.html": getMonthProxyToOrderBy
    };

    function getDateTimeFormat(field) {
        var format;
        var dateFormat = field['datetime-dateFormat'] || Util.DATE_FORMAT;
        var timeFormat = field['datetime-timeFormat'] || Util.TIME_FORMAT;
        switch (field['datetime-preset']) {
            case 'date':
            case 'month':
                format = dateFormat;
                break;
            case 'time':
                format = timeFormat;
                break;
            case 'datetime':
            case 'datetime-local':
                format = dateFormat + ' ' + timeFormat;
                break;
            default:
                format = dateFormat + ' ' + timeFormat;
                break;
        }
        return format;
    }

    function getDatetimeProxyToOrderBy(columnName, field) {
        var format = getDateTimeFormat(field);
        return getProxyToOrderBy(columnName, format);
    }

    function getDateProxyToOrderBy(columnName, field) {
        return getProxyToOrderBy(columnName, field.dateFormat || Util.DATE_FORMAT);
    }

    function getMonthProxyToOrderBy(columnName, field) {
        return getProxyToOrderBy(columnName, field.dateFormat || Util.DATE_FORMAT);
    }

    function getProxyToOrderBy(columnName, format) {
        var proxyProperty = getProxyProperty(columnName);
        buildDateProxyColumn(proxyProperty, columnName, format);
        return proxyProperty;
    }

    function getProxyProperty(columnName) {
        return "__proxy_" + columnName;
    }

    function buildDateProxyColumn(proxyProperty, sourceColumn, format) {
        if ($scope.widget.dataSource && Util.isArray($scope.widget.dataSource.data)) {
            $scope.widget.dataSource.data.forEach(function(item){
                item[proxyProperty] = moment(item[sourceColumn], format)._d;
            });
        }
    }

    function addColumnToOrder(columnName, reverse) {
        var field = $scope.widget.fields.filter(function(field){
            return field.name === columnName;
        }).shift();

        if (field) {
            var template = field.getTemplateSimpleName();
            if (orderByFieldFlow[template]) {
                columnName = orderByFieldFlow[template](columnName, field);
            }
        }

        orderByColumns.push((reverse ? '-' : '') + columnName);
    }

    function getOrderByColumns() {
        orderByColumns = [];

        if ($scope.widget.groups) {
            $scope.widget.groups.forEach(function(group, index) {
                addColumnToOrder(group.name);
            });
        }

        if ($scope.widget.order && !Util.isFunction($scope.widget.order)) {
            Util.parseToArray($scope.widget.order).forEach(function(collum){
                addColumnToOrder(collum, $scope.widget.reverse);
            });
        }
    }
    // END: Order by

    // BEGIN: Group and summarization
    $scope.fieldOrder = function(field) {
        return (field.fixed ? '0' : '1') + ('000'+field.sequenceInList).substr(-4);
    };

    $scope.widget.groupedData = [];

    // initialize widget.groups
    updateWidgetGroups();
    $scope.$on("updateWidgetGroups", function(){
        updateWidgetGroups();
    });

    function updateWidgetGroups() {
        $scope.widget.groups = $scope.widget.fields.filter(function(item) {
            if (item.grouped) item.isVisible = false;
            return item.aclIsVisible && item.grouped;
        }).sort(function(a, b) { return a.sequenceInList - b.sequenceInList; });
    }

    var oldItemsPerPage = $scope.widget.itemsPerPage;
    var oldHasPagination = $scope.widget.hasPagination;

    $scope.$watchCollection('widget.groups', function(newValue, oldValue) {
        if (newValue != oldValue) {
            if (newValue && newValue.length >= 1) {
                $scope.widget.itemsPerPage = 0;
                $scope.widget.hasPagination = false;
            } else {
                $scope.widget.itemsPerPage = oldItemsPerPage;
                $scope.widget.hasPagination = oldHasPagination;
            }
            $scope.widget._gridResize = true; // forces the grid element to recalculate column widths
            $scope.widget.reload();
        }
    });

    /** Watch Dashboard ComparePeriods Controller */
    $scope.$watchCollection('widget.dataSource.data', function() {
        updateGroupedData();
        self.periods = {};
        updatePeriods();
    });

    var groups = [];
    var summaryFields = [];
    var summary = {};

    function resetFooterVariables(group) {
        group.footer.count = 0;
        summaryFields.forEach(function(field) {
            group.footer[field.name] = {};
            group.footer[field.name].sum = 0;
            group.footer[field.name].count = 0;
        });
    }

    function addGroupHeader(group, index) {
        var groupHeaderRow = {
            __group: true,
            __groupHeader: true,
            __groupIndex: index,
            __groupLabel: ScreenService.i18n(group.label),
            __groupValue: !group.field.mask ? group.lastValue : Util.applyMask(group.lastValue, group.field.mask, maskEngine, ZhUtil),
            __groupOpened: true,
            __groupHeaders: []
        };
        // add header for outer groups to the added row
        for (var i = 0; i < index; i++) {
            groupHeaderRow.__groupHeaders.push(groups[i].lastGroupHeaderRow);
        }
        group.lastGroupHeaderRow = groupHeaderRow;
        $scope.widget.groupedData.push(groupHeaderRow);
    }

    function summaryData(group, row, isGroup) {
        var field, value, lastValue, _group;
        for (var i = 0; i < summaryFields.length; i++) {
            field = summaryFields[i];
            if (isGroup) {
                lastValue = ' (' + (!group.field.mask ? group.lastValue : Util.applyMask(group.lastValue, group.field.mask, maskEngine, ZhUtil)) + ')';
                _group = group.footer;
            } else {
                lastValue = ' (' + ScreenService.i18n('TOTAL') + ')';
                _group = group;
            }

            if (field.expression == 'COUNT') {
                row[field.name] = ScreenService.i18n('Count') + lastValue + ': ' + _group[field.name].count;
            } else if (field.expression == 'SUM') {
                value = _group[field.name].sum;
                row[field.name] = ScreenService.i18n('Sum') + lastValue + ': ' + (!field.mask ? value : Util.applyMask(value, field.mask, maskEngine, ZhUtil));
            } else if (field.expression == 'AVG') {
                value = _group[field.name].sum / _group[field.name].count;
                row[field.name] = ScreenService.i18n('Average') + lastValue + ': ' + (!field.mask ? value : Util.applyMask(value, field.mask, maskEngine, ZhUtil));
            }
        }
    }

    function addGroupFooters(groupIndex) {
        if (summaryFields.length === 0) return;

        for (var i = groups.length - 1; i >= groupIndex; i--) {
            if (groups[i].footer.count > 0) {
                var groupFooterRow = {
                    __group: true,
                    __groupFooter: true,
                    __groupIndex: i,
                    __groupHeaders: []
                };

                // add header for outer groups
                for (var g = 0; g < i; g++) {
                    groupFooterRow.__groupHeaders.push(groups[g].lastGroupHeaderRow);
                }

                summaryData(groups[i], groupFooterRow, true);

                $scope.widget.groupedData.push(groupFooterRow);
            }
        }
    }

    function addSummary() {
        if (summaryFields.length === 0 || groups.length === 0) return;

        var summaryRow = {
            __group: true,
            __groupFooter: true,
            __groupSummary: true
        };

        summaryData(summary, summaryRow);

        $scope.widget.groupedData.push(summaryRow);
    }

    function calcSummary(row) {
        groups.forEach(function(group) {
            group.footer.count += 1;
            if (summaryFields.length > 0) {
                summaryFields.forEach(function(field) {
                    if (!Util.isUndefined(row[field.name])) {
                        group.footer[field.name].count += 1;
                    }
                    if ((field.expression == 'SUM' || field.expression == 'AVG') && !isNaN(row[field.name])) {
                        group.footer[field.name].sum += parseFloat(row[field.name]);
                    }
                });
            }
        });

        summaryFields.forEach(function(field) {
            if (Util.isUndefined(summary[field.name])) {
                summary[field.name] = {
                    count: 0,
                    sum: 0
                };
            }

            if (!Util.isUndefined(row[field.name])) {
                summary[field.name].count += 1;
            }
            if ((field.expression == 'SUM' || field.expression == 'AVG') && !isNaN(row[field.name])) {
                summary[field.name].sum += parseFloat(row[field.name]);
            }
        });
    }

    function initSummaryFields() {
        summaryFields = $scope.widget.fields.filter(function(field) {
            return !!field.expression && !field.grouped;
        });

        // reset summary variable
        summary = {};
    }

    function initGroups() {
        groups = [];
        $scope.widget.groups.forEach(function(group, index) {
            var newGroup = {
                field: group,
                name: group.name,
                label: group.label,
                lastValue: undefined,
                footer: {}
            };
            resetFooterVariables(newGroup);

            groups.push(newGroup);
        });
    }

    function getOrderedDataByColumns(dataSet, orderByColumns) {
        return orderByColumns.length ? $filter('orderBy')(dataSet, orderByColumns) : dataSet;
    }

    function updateGroupedData() {
        if (Util.isUndefined($scope.widget.dataSource) || Util.isEmptyOrBlank($scope.widget.dataSource.data)) {
            $scope.widget.groupedData = [];
        } else {
            getOrderByColumns();

            if (Util.isUndefined($scope.widget.groups) || $scope.widget.groups.length === 0) {
                $scope.widget.groupedData = getOrderedDataByColumns($scope.widget.dataSource.data.filter(self.search), orderByColumns);
            } else {
                $scope.widget.groupedData = [];

                initSummaryFields();
                initGroups();

                var dataAux = getOrderedDataByColumns($scope.widget.dataSource.data.filter(self.search), orderByColumns);
                dataAux.forEach(function(row, index) {
                    var groupBreak = false;
                    groups.forEach(function(group, index) {
                        if (group.lastValue != row[group.name] || groupBreak) {
                            if (!groupBreak) addGroupFooters(index);

                            group.lastValue = row[group.name];
                            resetFooterVariables(group);

                            addGroupHeader(group, index);
                            groupBreak = true;
                        }
                    });

                    row.__groupHeaders = groups.map(function(group) { return group.lastGroupHeaderRow; });
                    $scope.widget.groupedData.push(row);

                    calcSummary(row);
                });

                if ($scope.widget.groupedData.length > 0) {
                    addGroupFooters(0);
                    addSummary();
                }
            }
        }

        if (self._updateVisibleRows) {
            self._updateVisibleRows();
        }
    }

    this.toggleGroup = function(row) {
        row.__groupOpened = !row.__groupOpened;

        if (self._updateVisibleRows) {
            self._updateVisibleRows();
        }
    };

    this.getClosedStatus = function(row) {
        var status = false;
        if (row.__groupHeaders && row.__groupHeaders.length > 0) {
            status = row.__groupHeaders.some(function(group) {
                return group.__groupOpened === false;
            });
        }
        return status;
    };

    /** Dashboard-compare-periods */
    function updatePeriods() {
        if(!$scope.widget.groupedData.length){
            if(!Util.isEmptyOrBlank($scope.widget.parameters.drillDownFilters)){
                $scope.widget.parameters.period = $scope.widget.period;
            }
        }
        if(Array.isArray($scope.widget.parameters.period) && !$scope.widget.isReversePeriods){
            $scope.widget.isReversePeriods = true;
            $scope.widget.parameters.periodCompare = Util.clone($scope.widget.parameters.period);
            $scope.widget.parameters.periodCompare.reverse();
        }
        $scope.widget.fields.forEach(function (field) {
            $scope.widget.fieldGroups.forEach(function (group) {
                if($scope.widget.groupedData.length){
                    setOrderByCustomList();
                }
                $scope.widget.groupedData.forEach(function (row) {
                    self.getPeriodsFromField(field, row, group);
                });
                if(!$scope.widget.groupedData.length || (Array.isArray($scope.widget.parameters.period) && $scope.widget.groupedData.length === 1 && $scope.widget.groupedData[0].NMFILIAL === 'Total')){
                    $scope.widget.visbleData = 'NOT_DATA';
                } else if(Array.isArray($scope.widget.parameters.period)){
                    $scope.widget.visbleData = 'DEFAULT';
                } else {
                    $scope.widget.visbleData = 'ONE_PERIOD';
                }
            });
        });

        $scope.widget.groupedData.sort(function (firstRow, secondRow) {
            if (firstRow.__isTotalRow) {
                return 1;
            } else if(secondRow.__isTotalRow) {
                return -1;
            } else if ($scope.widget.zmartOrdenationComparePeriods === 'ASC') {
                return firstRow.ordenationValue - secondRow.ordenationValue;
            } else {
                return secondRow.ordenationValue - firstRow.ordenationValue;
            }
        });

        if($scope.widget.parameters.limitRows && $scope.widget.groupedData.length) {
            $scope.widget.groupedData = $scope.widget.groupedData.slice(0, $scope.widget.parameters.limitRows);
        }
    }

    $scope.orderByCustom = function() {
        if($scope.widget.zmartOrdenationComparePeriods === 'DESC'){
            return self.orderByCustomListDesc;
        } else {
            return self.orderByCustomListAsc;
        }
    };

    /** Dashboard-compare-periods */
    self.doDrillDown = function(row, periodLabel) {
        var period = periodLabel.split(' - ');
        if($scope.widget.actionOnClick === 'filter') {
            row.isDashboardComparePeriod = true;
            row.periodClicked = {start: period[0], end: period[1]};
            ViewController = ApplicationContext.ViewController;
            ViewController.filterDashboardByWidget($scope.widget, row);
        } else {
            $scope.widget.parameters.period = [{start: period[0], end: period[1]}];
            DrillDownController = ApplicationContext.DrillDownController;
            DrillDownController.openNextIndicatorView($scope.widget, row);
        }
    };

    var setOrderByCustomList = function(){
        $scope.widget.parameters.periodCompare.forEach(function(per, index){
            if (index) {
                self.orderByCustomListAsc.push('order' + index);
                self.orderByCustomListDesc.push('-order' + index);
            }
        });
    };

    this.getPeriodsFromField = function (field, row, group) {
        var periods = $scope.widget.parameters.periodCompare;
        var filteredKeys;
        if (Array.isArray(periods)) {

            var aux = [];
            periods.forEach(function (p) {
                var a = $scope.widget.periodsFact + '_' + p.start + '_' + p.end;
                aux.push(a);
            });
            filteredKeys = aux;

            filteredKeys = filteredKeys.map(function (key) {
                var keySplit = key.split('_');
                var toDate = keySplit.slice(-1).shift();
                var fromDate = keySplit.slice(-2, -1).shift();
                var per = keySplit[keySplit.length - 2] + ' - ' + keySplit[keySplit.length - 1];
                toDate = moment(toDate, 'L');
                fromDate = moment(fromDate, 'L');
                return {
                    name: keySplit[0],
                    untouchedName: key,
                    from: fromDate,
                    to: toDate,
                    per: per
                };
            });

            filteredKeys.forEach(function (key) {
                setPeriodLabelAndOrder(key);
            });

            filteredKeys.sort(function (firstValue, secondValue) {
                return firstValue.order - secondValue.order;
            });

            var periods = [];

            for (var i = 0; i < filteredKeys.length; i++) {
                var key = filteredKeys[i];
                var rawPercent;

                var period = getFieldCopy(field);

                period.name = key.untouchedName;
                period.label = key.label;
                period.per = key.per;

                var lastPeriod;
                var lastPeriodValue;
                var thisPeriodValue;
                var percent;

                if (i === 0) {
                    row[period.name + "_showBadge"] = false;
                    row[period.name] = !!row[period.name] ? row[period.name] : 0;
                } else if (i === 1) {
                    lastPeriod = periods[0];
                    lastPeriodValue = row[lastPeriod.name] ? row[lastPeriod.name] : 0;
                    thisPeriodValue = row[period.name] ? row[period.name] : 0;
                    percent = calculatePercentage(lastPeriodValue, thisPeriodValue);
                    rawPercent = percent;

                    if($scope.widget.zmartOrdenationComparePeriods){

                        row['order1'] = parseFloat(percent);
                        if (row.__isTotalRow && $scope.widget.zmartOrdenationComparePeriods === 'ASC') {
                            row['order1'] = 1/0;
                        }
                        if (row.__isTotalRow && $scope.widget.zmartOrdenationComparePeriods === 'DESC') {
                            row['order1'] = -1/0;
                        }
                    } else {
                        if (row.__isTotalRow) {
                            row['order1'] = 1/0;
                        } else {
                            $scope.widget.dataSource.data.some(function(data, index){
                                if(data[group.name] === row[group.name]){
                                    row['order1'] = index;
                                }
                            });
                        }
                    }


                    row[period.name + "_isGreaterThanBaseValue"] = percent > 0;
                    row[period.name + "_isLowerThanBaseValue"] = percent < 0;
                    row[period.name + "_equalsBaseValue"] = percent === 0;
                    row[period.name + "_showBadge"] = true;

                    percent = Math.abs(percent);
                    percent = percent.toFixed(1);
                    row[period.name + "_percent"] = percent;
                    row[period.name] = !!row[period.name] ? row[period.name] : 0;
                } else {
                    lastPeriod = periods[0];
                    lastPeriodValue = row[lastPeriod.name] ? row[lastPeriod.name] : 0;
                    thisPeriodValue = row[period.name] ? row[period.name] : 0;
                    percent = calculatePercentage(lastPeriodValue, thisPeriodValue);
                    rawPercent = percent;

                    if($scope.widget.zmartOrdenationComparePeriods){
                        row[self.orderByCustomListAsc[i-1]] = parseFloat(percent);
                    }

                    row[period.name + "_isGreaterThanBaseValue"] = percent > 0;
                    row[period.name + "_isLowerThanBaseValue"] = percent < 0;
                    row[period.name + "_equalsBaseValue"] = percent === 0;
                    row[period.name + "_showBadge"] = true;

                    percent = Math.abs(percent);

                    percent = percent.toFixed(1);
                    row[period.name + "_percent"] = percent;
                    row[period.name] = !!row[period.name] ? row[period.name] : 0;
                }

                periods.push(period);
                if (row[group.name] === "Total") {
                    row.__isTotalRow = true;
                }

                if (i === filteredKeys.length - 1) {
                    row.__periodPercentRawValue = rawPercent;
                }
            }

        }

        self.periods[row[group.name]] = periods;
        return periods;
    };

    function calculatePercentage(lastPeriodValue, thisPeriodValue){
        if(Math.floor(lastPeriodValue) === 0 && Math.floor(thisPeriodValue) === 0){
            return 0;
        } else if(Math.floor(lastPeriodValue) === 0 && Math.floor(thisPeriodValue) !== 0) {
            return -100;
        } else {
            return (((lastPeriodValue * 100) / thisPeriodValue) - 100);
        }
    }

    function setPeriodLabelAndOrder(key) {
        var periods = $scope.widget.parameters.periodCompare;
        periods.some(function (p) {
            if(key.from._i === p.start && key.to._i === p.end){
                key.label = p.periodLabel ? p.periodLabel : p.start + ' - ' + p.end;
                return key;
            }
        })
    }

    function getFieldCopy(field) {
        return {
            name: field.name,
            label: field.label,
            aclIsVisible: field.aclIsVisible,
            aclReadOnly: field.aclReadOnly,
            align: field.align,
            class: field.class,
            direction: field.direction,
            events: field.events,
            function: field.function,
            idGenerated: field.idGenerated,
            isDate: field.isDate,
            isFact: field.isFact,
            isGenerated: field.isGenerated,
            isKey: field.isKey,
            isSortable: field.isSortable,
            isVisible: field.isVisible,
            mask: field.mask,
            readOnly: field.readOnly,
            readOnlyOnEdit: field.readOnlyOnEdit,
            sequenceInList: field.sequenceInList,
            showOnForm: field.showOnForm,
            showOnList: field.showOnList,
            showOnView: field.showOnView,
            showSeparatorButton: field.showSeparatorButton,
            showSignalButton: field.showSignalButton,
            translate: field.translate
        };
    }
}