function OperatorController(ScreenService, OperatorRepository, templateManager, ACLUtilities,
    OperatorService, LibUtilities, AclMessages, localStorageEngine, ZHPromise, General){
    
    var self = this;
    
    /* --------------------------
     *------ Get Widgets ------
     *---------------------------
     */
     
    var getOperatorWidget = function() {
        return templateManager.container.getWidget('OPERATOR');
    };
        
    var getAccessProfileWidget = function() {
        return templateManager.container.getWidget('ACCESS_PROFILE');
    };
    
    var getTabbedAddAccessprofileWidget = function() {
        return templateManager.container.getWidget('TABBED_ADD_ACCESS_PROFILE');
    };

    var getTabbedChildrenAddAccessprofileWidget = function() {
        return templateManager.container.getWidget('TABBED_CHILDREN_ADD_ACCESS_PROFILE');
    };

    var getAddAccessprofileWidget = function() {
        return templateManager.container.getWidget('ADD_ACCESS_PROFILE');
    };

    var getTabbedAddRelatedBranchWidget = function() {
        return templateManager.container.getWidget('TABBED_ADD_RELATED_BRANCH');
    };

    var getTabbedChildrenAddRelatedBranchWidget = function() {
        return templateManager.container.getWidget('TABBED_CHILDREN_ADD_RELATED_BRANCH');
    };

    var getAddRelatedBranchWidget = function() {
        return templateManager.container.getWidget('ADD_RELATED_BRANCH');
    };

    var getAddRelatedOrganizationWidget = function() {
        return templateManager.container.getWidget('ADD_RELATED_ORGANIZATION');
    };
    
    var getEditPasswordWidget = function(){
        return templateManager.container.getWidget('EDIT_PASSWORD');
    };
    
    var getRelatedBranchesWidget = function(){
        return templateManager.container.getWidget('RELATED_BRANCHES');
    };
    
    var getRelatedOrganizationsWidget = function(){
        return templateManager.container.getWidget('RELATED_ORGANIZAIONS');
    };
    /*---------------------------
     *------ Get Fields ---------
     *---------------------------
     */
    
    var getFieldCpfFromOperatorWidget = function() {
        return getOperatorWidget().getField('NRINSCRICAOPARC');
    };
    
    var getFieldOperatorCodeFromOperatorWidget = function() {
        return getOperatorWidget().getField('CDOPERADOR');
    };
    
    var getFieldAccessProfileFromAccessProfileWidget = function() {
        return getOperatorWidget().getField('NMPERFILACESSO');
    };
    
    var getFieldNewPasswordFromOperatorWidget = function(){
        return getOperatorWidget().getField('NEW_PASSWORD');
    };
    
    var getFieldConfirmPasswordFromOperatorWidget = function(){
        return getOperatorWidget().getField('CONFIRM_PASSWORD');
    };
    
    var getFieldAccessNameFromOperatorWidget = function(){
        return getOperatorWidget().getField('NMACESSO');
    };
    
    var getFieldEmailFromOperatorWidget = function(){
        return getOperatorWidget().getField('DSEMAILOPER');
    };

    var getCdFilialFromOperatorWidget = function(){
        return getOperatorWidget().getField('CDFILIAL');
    };

    var getNmOrganizationOperFromOperatorWidget = function(){
        return getOperatorWidget().getField('NMORGANIZACAOOPER');
    };
    
    var getFieldNmEstruturamFromOperatorWidget = function (){
        return getOperatorWidget().getField('NMESTRUTURAM');
    };

    /*-------------------------------------
     *-------- Generic Functions-----------
     *-------------------------------------
     */
    
    var passwordValidate = function(newPassword, confirmPassword){
        if(newPassword != confirmPassword) {
			ScreenService.showMessage(AclMessages.ALE_PASSWORD_NOT_MATCH, "ALERT");
			return false;
        }
		if(newPassword.length < 6 || newPassword.length  > 20) {
			ScreenService.showMessage(AclMessages.ALE_PASSWORD_INVALID_LENGTH, "ALERT");
			return false;
		}
		return true;
    };
    
    var toLowerCase = function(field){
        field.setValue(field.value().toLowerCase());
    };
    
    /*-------------------------------------
     *-------- Operator Widget ------------
     *-------------------------------------
     */
    
    /*---- Operator Widget Events ---------*/
     
    this.operatorWidgetOnEnter = function(args){
        var loginPromise = localStorageEngine.getLocalVar('loginPromise') || ZHPromise.when();
        ScreenService.showLoader();
        loginPromise.then(function() {
            OperatorService.getOrganization().then(function(response){
                if(response && response.dataset){
                    var org = response.dataset.findOrganization[0];
                    localStorageEngine.setLocalVar('organizationId', org.NRORG);
                    localStorageEngine.setLocalVar('organizationName', org.NMORGANIZACAO);
                }
            });
            ScreenService.hideLoader();
            var widget = getOperatorWidget();
            if(widget.calledByBackAction) {
                widget.calledByBackAction = false;
            }
            widget.currentRow = { IDOPERATIVO: 'A' };
            ACLUtilities.applyDefaultFilterOnEnter(widget);
        }).catch(ScreenService.hideLoader);
    };
    
    this.operatorWidgetAfterMoveRow = function(args){
        var widget = getOperatorWidget();
        
        //If current operator doesn't have a business partnership, create it
        if(Util.isEmptyOrBlank(widget.currentRow.NRPARCNEGOCIO) && !widget.currentRow.__is_new) {
            widget.currentRow.CDTIPOINSCRICAO = getInscriptionType(widget.currentRow.NRINSCRICAOPARC);
            widget.currentRow.changed = true;
            OperatorService.createBusinessPartnership(widget.currentRow).then(function (businessPartnership) {
                widget.currentRow.NRPARCNEGOCIO = businessPartnership.NRPARCNEGOCIO;
                widget.currentRow.NMPARCNEGOCIO = businessPartnership.NMPRINCIPALPARC;
                localStorageEngine.setLocalVar('businessPartnership', widget.currentRow.NRPARCNEGOCIO);    
            });
        } else {
            localStorageEngine.setLocalVar('businessPartnership', widget.currentRow.NRPARCNEGOCIO);    
        }
        
        localStorageEngine.setLocalVar('operatorCode', widget.currentRow.CDOPERADOR);
        localStorageEngine.setLocalVar('operatorEmail', widget.currentRow.DSEMAILOPER);
        this.setLoggedOperatorInfos();
        getAccessProfileWidget().reload();
    };
    
    this.operatorWidgetOnEnterOnce = function(args){
        var loginPromise = localStorageEngine.getLocalVar('loginPromise') || ZHPromise.when();
        ScreenService.showLoader();
        loginPromise.then(function() {
            ScreenService.hideLoader();
            self.setOperatorWindowMode(window.showFullOperatorWindow, window.filialRuleInFullOperatorWindow);
        }).catch(ScreenService.hideLoader);
    };

    this.operatorFieldOnChangeCdOperador = function () {
        var widget = getOperatorWidget();
        var codeOperator = widget.currentRow.CDOPERADOR;
        if(!isNaN(codeOperator)){
            var value = General.fill(codeOperator, 12);
            widget.currentRow.CDOPERADOR = value;
        }
    };
    
    /*---- Operator Widget Actions --------*/
    
    this.operatorActionEventBackView = function(args){
        ScreenService.closeSwipe();
        getOperatorWidget().calledByBackAction = true;
    };
    
    this.operatorActionEventCancel = function(args){
        cancelWithoutReloadMasterDetail(getOperatorWidget());
        getFieldOperatorCodeFromOperatorWidget().readOnly = true;
        setInvisibleAndRequiredPasswordField(false);
        setEnableTabWidgetsOperator(true);
        getFieldOperatorCodeFromOperatorWidget().validations = {required: {}};
        getOperatorWidget().cdoperadorValidation = false;
    };
    
    var cancelWithoutReloadMasterDetail = function(widget){
        if(widget.currentRow.__is_new){
            ScreenService.closeSwipe();
            widget.dataSource.data.pop();
        } else {
            widget.dataSource.data.some(function (row, index) {
                if (row.CDOPERADOR == widget.backupRow.CDOPERADOR) {
                    widget.dataSource.data[index] = widget.backupRow;
                    return true;
                }
            });
            widget.currentRow = widget.backupRow;
            widget.view();
        }
    };
    
    this.operatorActionEventAdd = function(args){
        var widget = getOperatorWidget();
        var nmOrganizationOperField = getNmOrganizationOperFromOperatorWidget();
        var operatorCodeField = getFieldOperatorCodeFromOperatorWidget();

        OperatorService.getParamGeral().then(function (response) {
            if(response.dataset.paramgeral[0].IDGERACDOPERADOR === 'S'){
                operatorCodeField.readOnly = true;
                operatorCodeField.validations = {};
                widget.cdoperadorValidation = false;
            } else {
                operatorCodeField.readOnly = false;
                operatorCodeField.validations = {required: {}};
                widget.cdoperadorValidation = true;
            }

            widget.newRow();
            widget.currentRow.NMPAIS = "Brasil";
            widget.currentRow.CDPAIS = "0055";

            nmOrganizationOperField.reload().then(function(data){
                setReadOnlyFieldNmOrganizationOper(data, widget, true);
            });

            setInvisibleAndRequiredPasswordField(true);
            getFieldNmEstruturamFromOperatorWidget().reload();
            OperatorService.getOrganization().then(function(response){
                if(response && response.dataset){
                    var org = response.dataset.findOrganization[0];
                    widget.currentRow.NMORGANIZACAO = org.NMORGANIZACAO;
                }
            });
        });
        this.setLoggedOperatorInfos();
        setEnableTabWidgetsOperator(false);
    };
    
    this.operatorActionEventEdit = function(args){
        var widget = getOperatorWidget();
        var nmOrganizationOperField = getNmOrganizationOperFromOperatorWidget();
        widget.backupRow = Util.clone(widget.currentRow);
        setInvisibleAndRequiredPasswordField(false);
        setEnableTabWidgetsOperator(false);
        nmOrganizationOperField.reload().then(function(data){
            setReadOnlyFieldNmOrganizationOper(data, widget);
            widget.edit();
        });
    };

    var setReadOnlyFieldNmOrganizationOper = function(data, widget, isNew){
        isNew = isNew || false;
        var nmOrganizationOperField = getNmOrganizationOperFromOperatorWidget();
        if(data.dataset.findOrgOper.length < 2){
            if(data.dataset.findOrgOper.length > 0){
                var dataset = data.dataset.findOrgOper[0];
                widget.currentRow.NMORGANIZACAOOPER = dataset.NMORGANIZACAOOPER;
                widget.currentRow.NRORGOPER = dataset.NRORG;
            }
            nmOrganizationOperField.readOnly = true;
        } else {
            nmOrganizationOperField.readOnly = false;
            if (isNew) {
                widget.currentRow.NMORGANIZACAOOPER = localStorageEngine.getLocalVar('organizationName');
                widget.currentRow.NRORGOPER = localStorageEngine.getLocalVar('organizationId');
            }
        }
    };
    
    this.operatorActionEventRemove = function(args){
        ScreenService.confirmMessage(AclMessages.CON_DEL_OPERATOR , 'question', function () {
            ACLUtilities.genericDeleteEvent(getOperatorWidget()).then(function () {
            	ScreenService.successNotification(AclMessages.SUC_DEL_OPERATOR);
            }).catch(LibUtilities.showErrorMessage);
        });
    };

    var validateCodeOperator = function(codeOperator, operatorWidget){
        return codeOperator.length !== 12 && operatorWidget.currentRow.__is_new && operatorWidget.cdoperadorValidation;
    };

    this.operatorActionEventSave = function(args){
        var widget = getOperatorWidget();
        var cpf = ACLUtilities.getOnlyNumberInString(widget.currentRow.NRINSCRICAOPARC);
        var codeOperator = widget.currentRow.CDOPERADOR;
        if (validateCodeOperator(codeOperator, widget)){
            ScreenService.showMessage(AclMessages.ALE_MIN_LENGTH_OPERATOR_CODE);
            return false;
        }
        if (ACLUtilities.checkRequirements(widget) && ACLUtilities.validateCPF(cpf)) {
            
            widget.currentRow.CDTIPOINSCRICAO = getInscriptionType(cpf);
            widget.currentRow.IDOPERATIVO = (widget.currentRow.IDATIVO == 'S') ? 'A' : 'I';
            widget.currentRow.NRINSCRICAOPARC = cpf;
                
            if(!widget.currentRow.__is_new){
                widget.currentRow.OLDNRINSCRICAOPARC = widget.backupRow.NRINSCRICAOPARC;
                widget.currentRow.OLDCDCLIENTE = widget.backupRow.CDCLIENTE;
                widget.currentRow.OLDNMOPERADOR = widget.backupRow.NMOPERADOR;
                widget.currentRow.changed = true;
            }else{
                widget.currentRow.ADMINISTRATOR = widget.ADMINISTRATOR;
                widget.currentRow.BRANCHES = widget.BRANCHES;
            }
            
            OperatorService.saveOperator(widget.currentRow).then(function(response){
                var operatorCode = response.dataset.operator[0].CDOPERADOR;
                    var rowInDataSource = ACLUtilities.getRowInDataSource(widget.dataSource, 'CDOPERADOR', response.dataset.operator[0].CDOPERADOR);
                    widget.view();
                    var filter = [
                        {
                            "name": "CDOPERADOR",
                            "value": operatorCode,
                            "operator": "="
                        }
                    ];
                    
                    ACLUtilities.setDataSourceAfterSave(widget, filter, "CDOPERADOR").then(function(row){
                        widget.setCurrentRow(row);
                        localStorageEngine.setLocalVar('businessPartnership', response.dataset.operator[0].NRPARCNEGOCIO);
                        getFieldOperatorCodeFromOperatorWidget().readOnly = true;
                        widget.currentRow.__is_new = false;
                        setInvisibleAndRequiredPasswordField(false);
                        ScreenService.successNotification(AclMessages.SUC_SAVE_OPERATOR);
                        setEnableTabWidgetsOperator(true);
                        getFieldOperatorCodeFromOperatorWidget().validations = {required: {}};
                        getOperatorWidget().cdoperadorValidation = false;
                    });
                    
            }).catch(LibUtilities.showErrorMessage);
        }
    };
    
    /*------ Operator Widget auxiliary methods ------*/
    
    var setInvisibleAndRequiredPasswordField = function(status){
        if(status){
            getFieldNewPasswordFromOperatorWidget().showOnForm = true;
            getFieldConfirmPasswordFromOperatorWidget().showOnForm = true;
            getFieldNewPasswordFromOperatorWidget().validations = {required: {}};
            getFieldConfirmPasswordFromOperatorWidget().validations = {required: {}};
            
        } else {
            getFieldNewPasswordFromOperatorWidget().showOnForm = false;
            getFieldConfirmPasswordFromOperatorWidget().showOnForm = false;
            getFieldNewPasswordFromOperatorWidget().validations = false;
            getFieldConfirmPasswordFromOperatorWidget().validations = false;
        }
    };
    
    var getInscriptionType = function (inscription) {
        if(inscription) {
            if(inscription.length == 11) {
                return "CPF";
            }else if(inscription.length == 14) {
                return "CNPJ";
            }
        } else {
            return "LIVRE";
        }
    };
    
    this.accessNameFieldFromOperatorWidgetFieldOnChange = function(args){
        toLowerCase(getFieldAccessNameFromOperatorWidget());
    };
    
    this.emailFieldFromOperatorWidgetFieldOnChange = function(args){
        toLowerCase(getFieldEmailFromOperatorWidget());
    };
    
    /**
     *  Use project's 'showFullOperatorWindow' and 'filialRuleInFullOperatorWindow' parameters to
     * set fields and tabs visibility.
     * 
     *  This parameter has to be setted to the 'window' at 'environment.js' as true or false
    */
    this.setOperatorWindowMode = function(showFullOperatorWindow, filialRuleInFullOperatorWindow){
        showFullOperatorWindow         = showFullOperatorWindow         || false;
        filialRuleInFullOperatorWindow = filialRuleInFullOperatorWindow || false;
        var operatorWidget = getOperatorWidget();
        var filialField = getCdFilialFromOperatorWidget();
        
        if(showFullOperatorWindow){
            operatorWidget.COMPLETEMODE = true;
            this.setVisibilityByMode(operatorWidget, true);
            if(filialRuleInFullOperatorWindow){    
                this.checkIfIsAdm().then(function(adm){
                    if(adm === true){
                        operatorWidget.ADMINISTRATOR = true;
                        filialField.filterProperties.isVisible = true;
                        filialField.isVisible = false;
                    }else{
                        this.setOperatorBranchesInOperatorWidget(operatorWidget);
                    }
                }.bind(this)).catch(function(adm){
                    operatorWidget.ADMINISTRATOR = false;    
                    filialField.filterProperties.isVisible = false;
                    filialField.isVisible = false;
                    this.setOperatorBranchesInOperatorWidget(operatorWidget);
                }.bind(this));
            }
        }else{
            operatorWidget.COMPLETEMODE = false;
            this.setVisibilityByMode(operatorWidget, false);
            filialField.filterProperties.isVisible = false;
            filialField.isVisible = false;
        }
        
    };
    
    this.setOperatorBranchesInOperatorWidget = function(widget){
        var filter = [
            {
                "name": "CDOPERADOR",
                "value": localStorageEngine.getLocalVar('userId') || localStorageEngine.getLocalVar('CDOPERADOR'),
                "operator": "="
            }
        ];
        OperatorService.getOperatorBranches(filter).then(function(response){
            if(response && response.dataset.filioperCompleto){
                var branches = response.dataset.filioperCompleto.map(function(branch){return branch.CDFILIAL;});
                widget.BRANCHES = branches;
                
                if(branches.length <= 0) return;
                /* Filter Related Branch tab */
                var branchWidget = getAddRelatedBranchWidget();
                var filterApplied = false;
                branchWidget.dataSourceFilter.forEach(function(filter){
                    if(filter.name === 'CDFILIAL') {
                        filterApplied = true;
                    }
                });
                if(!filterApplied){
                    branchWidget.dataSourceFilter.push({
                        name:"CDFILIAL",
                        operator:"IN",
                        value:branches
                    });
                    getCdFilialFromOperatorWidget().dataSourceFilter.push({
                        name: "CDFILIAL",
                        operator: "IN",
                        value: branches
                    });
                    getOperatorWidget().dataSourceFilter.push({
                        name: "CDFILIAL",
                        operator: "IN",
                        value: branches
                    });
                }
            }    
        });
    };
    
    this.setVisibilityByMode = function(operatorWidget, visible){
        operatorWidget.fields.forEach(function(field){
            if(field.visibleOnlyInCompleteMode === true){
                field.isVisible = visible;
            }
        });
        
        operatorWidget.widgets.forEach(function(widget){
            if(widget.visibleOnlyInCompleteMode === true){
                widget.isVisible = visible;
            }
        });
    };
    
    /**
     * Apply Branch Filter to the widget if the logged operator
     * isn't an administrator
    */
    this.operatorWidgetAfterApply = function(args){
        ACLUtilities.setFilterInit(getOperatorWidget());
        var operatorWidget = getOperatorWidget();
        if(operatorWidget.ADMINISTRATOR === false){
            var filterApplied = false;
            operatorWidget.dataSourceFilter.forEach(function(filter) {
                if(filter.name === 'CDFILIAL') {
                    filterApplied = true;
                }
            });
            if(!filterApplied && operatorWidget.BRANCHES.length > 0){
                operatorWidget.dataSourceFilter.push({
                    name:"CDFILIAL",
                    operator:"IN",
                    value:operatorWidget.BRANCHES
                });
            }
        }
    };
    
    /**
     * Check if the logged operator is Administrator
    */
    this.checkIfIsAdm = function(){
        var promise = ZHPromise.defer();
        var accessProfiles = localStorageEngine.getLocalVar('accessProfiles');
        if(accessProfiles){
            OperatorService.checkIfIsAdm(accessProfiles).then(function(response){ 
                if(response.dataset && response.dataset.administrator){
                    if(response.dataset.administrator[0] === true){
                        promise.resolve(true);
                    }else{
                        promise.resolve(false);    
                    }
                    localStorageEngine.setLocalVar('loggedOperatorNrorg', response.dataset.administrator[1]);
                }else{
                    promise.reject(false);
                }
            });
        }else{
            promise.reject(false);
        }
        return promise.promise;
    };
    
    /**
     *   Set the IDOPERMASTER's readonly based in the Logged Operator
     *  and save some of its infos in the widget to use later
    */
    this.setLoggedOperatorInfos = function(){
        var operatorWidget = getOperatorWidget();
        if(typeof operatorWidget.IDOPERMASTERLOGGED === "undefined"){
            OperatorRepository.getLoggedOperator().then(function(response){
                var operator = response[0];
                if(operator.IDOPERMASTER === 'S'){
                    operatorWidget.getField('IDOPERMASTER').readOnly = false;
                }else{
                    operatorWidget.getField('IDOPERMASTER').readOnly = true;
                }
                operatorWidget.IDOPERMASTERLOGGED = operator.IDOPERMASTER;
                operatorWidget.NRORGTRABLOGGED    = operator.NRORGTRAB;
            });
        }else{
            if(operatorWidget.IDOPERMASTERLOGGED === 'S'){
                operatorWidget.getField('IDOPERMASTER').readOnly = false;
            }else{
                operatorWidget.getField('IDOPERMASTER').readOnly = true;
            }    
        }
    };
    
    /**
     *      Check if the current Organization(relationship) is the Logged Operator's org.
     *  Only in this case he can edit/delete the relationship.
    */
    this.havePermissionToChangeOrg = function(){
        var loggedOrg = getOperatorWidget().NRORGTRABLOGGED;
        if(getRelatedOrganizationsWidget().currentRow.NRORGREL !== loggedOrg){
            ScreenService.showMessage(AclMessages.ALE_PERMISSION_ORGANIZATION);
        }else{
            return true;
        }
    };

    var setEnableTabWidgetsOperator = function(value) {
          var widget = getOperatorWidget();
          widget.widgets.forEach(function (widget){
              widget.enabled = value;
          });
    };
    
    /*-------------------------------------
     *------ Access Profile Widget --------
     *-------------------------------------
     */
     
    /*---- Access profile Widget Actions --------*/
    
    this.accessProfileActionEventBackList = function(args){
        ScreenService.closeSwipe();
    };
    
    this.accessProfileActionEventBackView = function(args){
        ScreenService.closeSwipe();
    };
    
    this.accessProfileActionEventCancel = function(args){
        ACLUtilities.genericCancelEvent(getAccessProfileWidget());
    };
    
    this.accessProfileActionEventAdd = function(args){
        var widget = getAddAccessprofileWidget();
        var tabbedAddAccessProfileWidget = getTabbedAddAccessprofileWidget();
        var tabbedChildrenAddAccessProfileWidget = getTabbedChildrenAddAccessprofileWidget();
        ScreenService.openSwipe(tabbedAddAccessProfileWidget).then(function () {
            tabbedAddAccessProfileWidget.edit();
            widget.activate();
        });
    };
    
    this.accessProfileActionEventEdit = function(args){
        getAccessProfileWidget().edit();  
    };
    
    this.accessProfileActionEventRemove = function(args){
        ScreenService.confirmMessage(AclMessages.CON_DEL_PROFILE , 'question', function () {
            ACLUtilities.genericDeleteEvent(getAccessProfileWidget()).then(function () {
            	ScreenService.successNotification(AclMessages.SUC_DELETE_ACCESS_PROFILE);
            }).catch(LibUtilities.showErrorMessage);
        });
    };
    
    this.accessProfileActionEventSave = function(args){
        var accessProfileWidget = getAccessProfileWidget();
        var currentRow = accessProfileWidget.currentRow;
        currentRow.__is_new = false;
        currentRow.changed = true;
        OperatorService.saveAccessProfileByOperator([currentRow]).then(function(response){
            var partnerAccessProfileNumber = response.dataset.accessProfileByOperator[0].NRPERFACESSPARC;
            ScreenService.successNotification(AclMessages.SUC_SAVE_ACCESS_PROFILE);
            accessProfileWidget.reload().then(function(){
                var rowInDataSource = ACLUtilities.getRowInDataSource(accessProfileWidget.dataSource, 'NRPERFACESSPARC', partnerAccessProfileNumber);
                if(rowInDataSource){
                    accessProfileWidget.setCurrentRow(rowInDataSource);
                    accessProfileWidget.view();
                } else {
                    ScreenService.closeSwipe();
                    accessProfileWidget.list(true);
                }   
            });
        }).catch(LibUtilities.showErrorMessage);
    };
    
    this.openPasswordEdit = function(args){
        ScreenService.openPopup(getEditPasswordWidget());
    };
    
    /*---- Access profile Widget Events --------*/
    
    this.accessProfileWidgetOnEnterTab = function(args){
        getAccessProfileWidget().reload();
    };
    
    /*-------------------------------------
     *---- Add Access Profile Widget ------
     *-------------------------------------
     */
    
    /*---- Add Access profile Widget Actions --------*/
    
    this.addAccessProfileActionEventCancel = function(args){
        getAddAccessprofileWidget().unSelectAll();
        getTabbedChildrenAddAccessprofileWidget().currentRow.DTVALIDADE = '';
        ScreenService.closeSwipe();
    };
    
    this.addAccessProfileActionEventSave = function(args){
        var addAccessprofileWidget = getAddAccessprofileWidget();
        var tabbedChildrenAddAccessprofileWidget = getTabbedChildrenAddAccessprofileWidget();
        var accessProfileWidget = getAccessProfileWidget();
        var checkedRows = addAccessprofileWidget.getCheckedRows();
        checkedRows.forEach(function(element){
            element.NRPARCNEGOCIO = localStorageEngine.getLocalVar('businessPartnership');
            element.DTVALIDADE = tabbedChildrenAddAccessprofileWidget.currentRow.DTVALIDADE || null;
            element.__is_new = true;
            element.changed = true;
        });
        
        OperatorService.saveAccessProfileByOperator(checkedRows).then(function(){
            accessProfileWidget.reload().then(function(){
                addAccessprofileWidget.unSelectAll();
                tabbedChildrenAddAccessprofileWidget.currentRow.DTVALIDADE = '';
                ScreenService.closeSwipe();
                accessProfileWidget.activate();
                ScreenService.successNotification(AclMessages.SUC_SAVE_ACCESS_PROFILE);
            });
        }).catch(LibUtilities.showErrorMessage);
    };
    
    /*---- Add Access profile Widget Events --------*/
    
    this.addAccessProfileWidgetOnEnter = function(){
        getAddAccessprofileWidget().reload();
    };
    
    /*-------------------------------------
     *---- Edit Password Widget ------
     *-------------------------------------
     */
    
    /*---- Edit password Widget Actions --------*/
    
    this.editPasswordActionEventCancel = function(args){
        ScreenService.closePopup(getEditPasswordWidget());
    };
    
    this.editPasswordActionEventSave = function(args){
        var editPasswordWidget = getEditPasswordWidget();
        if(editPasswordWidget.isValid()){
    		var newPassword     = editPasswordWidget.currentRow.NEW_PASSWORD;
    		var confirmPassword = editPasswordWidget.currentRow.CONFIRM_PASSWORD;
    		if(passwordValidate(newPassword, confirmPassword)){
    		    OperatorService.resetPassword(editPasswordWidget.currentRow).then(function(response){
                    ScreenService.successNotification(AclMessages.SUC_SAVE_PASSWORD);
                    ScreenService.closePopup();
			    }).catch(LibUtilities.showErrorMessage);
    		}
        } else {
            ScreenService.showMessage(AclMessages.ALE_FILL_REQ_FIELD);
        }
    };
    
    /*---- Edit password Widget Events --------*/
    
    this.editPasswordWidgetOnEnter = function(args){
        var widget = getEditPasswordWidget();
        widget.newRow();
        widget.currentRow.DSEMAILOPER = localStorageEngine.getLocalVar('operatorEmail');
        widget.currentRow.CDOPERADOR = localStorageEngine.getLocalVar('operatorCode');
    };
    
        
    /*-------------------------------------
     *------ Related Branches Widget --------
     *-------------------------------------
     */
     
    /*---- Related Branches Widget Actions --------*/
    
    this.relatedBranchesActionEventBackList = function(args){
        ScreenService.closeSwipe();
    };
    
    this.relatedBranchesActionEventBackView = function(args){
        ScreenService.closeSwipe();
    };
    
    this.relatedBranchesActionEventCancel = function(args){
        ACLUtilities.genericCancelEvent(getRelatedBranchesWidget());
    };
    
    this.relatedBranchesActionEventAdd = function(args){
        var widget = getAddRelatedBranchWidget();
        var tabbedAddRelatedBranch = getTabbedAddRelatedBranchWidget();
        var tabbedChildrenAddRelatedBranch = getTabbedChildrenAddRelatedBranchWidget();
        widget.reload();
        ScreenService.openSwipe(tabbedAddRelatedBranch).then(function () {
            tabbedAddRelatedBranch.edit();
            widget.activate();
        });
    };
    
    this.relatedBranchesActionEventEdit = function(args){
        getRelatedBranchesWidget().edit();  
    };
    
    this.relatedBranchesActionEventRemove = function(args){
        var widget = getRelatedBranchesWidget();
        ScreenService.confirmMessage(AclMessages.CON_DEL_BRANCH , 'question', function () {
            widget.remove().then(function () {
                ScreenService.successNotification(AclMessages.SUC_DELETE_RELATED_BRANCH);
                ScreenService.closeSwipe().then(function() {
                    widget.reload();
                });
            }).catch(LibUtilities.showErrorMessage);
        });
    };
    
    this.relatedBranchesActionEventSave = function(args){
        var relatedBranchesWidget = getRelatedBranchesWidget();
        var currentRow = relatedBranchesWidget.currentRow;
        currentRow.__is_new = false;
        currentRow.changed = true;
        OperatorService.saveRelatedBranchByOperator([currentRow]).then(function(response){
            var cdfilial = response.dataset.relatedBranchesByOperator[0].CDFILIAL;
            ScreenService.successNotification(AclMessages.SUC_SAVE_RELATED_BRANCH);
            relatedBranchesWidget.reload().then(function(){
                var rowInDataSource = ACLUtilities.getRowInDataSource(relatedBranchesWidget.dataSource, 'CDFILIAL', cdfilial);
                if(rowInDataSource){
                    relatedBranchesWidget.setCurrentRow(rowInDataSource);
                    relatedBranchesWidget.view();
                } else {
                    ScreenService.closeSwipe();
                    relatedBranchesWidget.list(true);
                }   
            });
        }).catch(LibUtilities.showErrorMessage);
    };
    
    /*---- Related Branches Widget Events --------*/
    
    this.relatedBranchesWidgetOnEnterTab = function(args){
        getRelatedBranchesWidget().reload();
    };
    
    /*-------------------------------------
     *---- Add Related Branch Widget ------
     *-------------------------------------
     */
    
    /*---- Add Related Branch Widget Actions --------*/
    
    this.addRelatedBranchActionEventCancel = function(args){
        getAddRelatedBranchWidget().unSelectAll();
        getTabbedChildrenAddRelatedBranchWidget().currentRow.DTVALIDADE = '';
        ScreenService.closeSwipe();
    };
    
    this.addRelatedBranchActionEventSave = function(args){
        var addRelatedBranchWidget = getAddRelatedBranchWidget();
        var tabbedChildrenAddRelatedBranchWidget = getTabbedChildrenAddRelatedBranchWidget();
        var relatedBranchWidget = getRelatedBranchesWidget();
        var checkedRows = addRelatedBranchWidget.getCheckedRows();
        checkedRows.forEach(function(element){
            element.CDOPERADOR = localStorageEngine.getLocalVar('operatorCode');
            element.DTVALIDADE = tabbedChildrenAddRelatedBranchWidget.currentRow.DTVALIDADE || null;
            element.__is_new = true;
            element.changed = true;
        });
        
        OperatorService.saveRelatedBranchByOperator(checkedRows).then(function(){
            relatedBranchWidget.reload().then(function(){
                addRelatedBranchWidget.unSelectAll();
                tabbedChildrenAddRelatedBranchWidget.currentRow.DTVALIDADE = '';
                ScreenService.closeSwipe();
                relatedBranchWidget.activate();
                ScreenService.successNotification(AclMessages.SUC_SAVE_RELATED_BRANCH);
            });
        }).catch(LibUtilities.showErrorMessage);
    };
    
    /*---- Add Related Branch Widget Events --------*/
    
    this.addRelatedBranchWidgetOnEnter = function(){
        getAddRelatedBranchWidget().reload();
    };
    
    /*-------------------------------------
     *------ Related Organizations Widget --------
     *-------------------------------------
     */
     
    /*---- Related Organizations Widget Actions --------*/
    
    this.relatedOrganizationsActionEventBackList = function(args){
        ScreenService.closeSwipe();
    };
    
    this.relatedOrganizationsActionEventBackView = function(args){
        ScreenService.closeSwipe();
    };
    
    this.relatedOrganizationsActionEventCancel = function(args){
        ACLUtilities.genericCancelEvent(getRelatedOrganizationsWidget());
    };
    
    this.relatedOrganizationsActionEventAdd = function(args){
        var widget = getAddRelatedOrganizationWidget();
        ScreenService.openPopup(widget);
    };
    
    this.relatedOrganizationsActionEventEdit = function(args){
        if(this.havePermissionToChangeOrg()){
            getRelatedOrganizationsWidget().edit();  
        }
    };
    
    this.relatedOrganizationsActionEventRemove = function(args){
        if(this.havePermissionToChangeOrg()){
            ScreenService.confirmMessage(AclMessages.CON_DEL_ORGANIZATION , 'question', function () {
                ACLUtilities.genericDeleteEvent(getRelatedOrganizationsWidget()).then(function () {
                	ScreenService.successNotification(AclMessages.SUC_DELETE_RELATED_ORGANIZATION);
                }).catch(LibUtilities.showErrorMessage);
            });
        }
    };
    
    this.relatedOrganizationsActionEventSave = function(args){
        var relatedOrganizationsWidget = getRelatedOrganizationsWidget();
        var currentRow = relatedOrganizationsWidget.currentRow;
        currentRow.__is_new = false;
        currentRow.changed = true;
        OperatorService.saveRelatedOrganizationByOperator([currentRow]).then(function(response){
            var nrorgrel = response.dataset.relatedOrganizationsByOperator[0].NRORGREL;
            ScreenService.successNotification(AclMessages.SUC_SAVE_RELATED_ORGANIZATION);
            relatedOrganizationsWidget.reload().then(function(){
                var rowInDataSource = ACLUtilities.getRowInDataSource(relatedOrganizationsWidget.dataSource, 'NRORGREL', nrorgrel);
                if(rowInDataSource){
                    relatedOrganizationsWidget.setCurrentRow(rowInDataSource);
                    relatedOrganizationsWidget.view();
                } else {
                    ScreenService.closeSwipe();
                    relatedOrganizationsWidget.list(true);
                }   
            });
        }).catch(LibUtilities.showErrorMessage);
    };
    
    /*---- Related Organizations Widget Events --------*/
    
    this.relatedOrganizationsWidgetOnEnterTab = function(args){
        getRelatedOrganizationsWidget().reload();
    };
    
    /*-------------------------------------
     *---- Add Related Organization Widget ------
     *-------------------------------------
     */
    
    /*---- Add Related Organization Widget Actions --------*/
    
    this.addRelatedOrganizationActionEventCancel = function(args){
        getAddRelatedOrganizationWidget().unSelectAll();
        ScreenService.closePopup();        
    };
    
    this.addRelatedOrganizationActionEventSave = function(args){
        var addRelatedOrganizationWidget = getAddRelatedOrganizationWidget();
        var relatedOrganizationWidget = getRelatedOrganizationsWidget();
        var checkedRows = addRelatedOrganizationWidget.getCheckedRows();
        checkedRows.forEach(function(element){
            element.CDOPERADOR = localStorageEngine.getLocalVar('operatorCode');
            element.__is_new = true;
            element.changed = true;
        });
        
        OperatorService.saveRelatedOrganizationByOperator(checkedRows).then(function(){
            relatedOrganizationWidget.reload().then(function(){
                addRelatedOrganizationWidget.unSelectAll();
                ScreenService.closePopup();
                relatedOrganizationWidget.activate();
                ScreenService.successNotification(AclMessages.SUC_SAVE_RELATED_ORGANIZATION);
            });
        }).catch(LibUtilities.showErrorMessage);
    };
    
    /*---- Add Related Organization Widget Events --------*/
    
    this.addRelatedOrganizationWidgetOnEnter = function(){
        getAddRelatedOrganizationWidget().reload();
    };
    
    this.checkshowAutoCadVendedor = function(args){
        args.owner.getWidget('OPERATOR').fieldGroups.forEach(function(group){
            if(group.name == 'SELLER') {
                group.isVisible = window.showAutoCadVendedor;
            }
        });
    };
    
}

Configuration(function(ContextRegister){
    ContextRegister.register('OperatorController', OperatorController);
});