<?php

namespace Teknisa\Model\Teknisa\Repositories;

use Doctrine\DBAL\Connection;
use Teknisa\Model\Teknisa\Entities;
use OrganizationsManagement\Util\Utilities; 

class OmProduto extends \Doctrine\ORM\EntityRepository {

    public function getDeploys($conditions, $limit, $offset){
        $query = <<<SQL
            SELECT *
              FROM (SELECT P.ID AS PRODUCT_ID, P.NAME AS PRODUCT_NAME, P.ORGANIZATION_ID AS PRODUCT_NRORG, S.ID AS SERVER_ID,
                           S.NAME AS SERVER_NAME, PS.PATH, R.EXTERNAL_CODE, CLI.NMRAZSOCCLIE, CLI.NMFANTCLIE, DP.NAME AS DEPLOY_NAME,
                           RC.NAME AS CREATED_BY_NAME, RU.NAME AS UPDATED_BY_NAME, CCC.DSCLASSIFICACAOCASO, NC.NMNATUCONT, MC.NMMOTICONT,
                           DECODE(PV.VERSION, NULL, '-', PV.VERSION || '.' || PV.REVISION  || '.' || PV.BUILD) AS DEPLOY_VERSION,
                           TO_DATE(DP.CREATED,'DD/MM/YYYY HH24:MI:SS') AS CREATED, DP.IS_ACTIVE, DP.CREATED_BY, DP.ID AS DEPLOY_ID,
                           TO_DATE(DP.UPDATED,'DD/MM/YYYY HH24:MI:SS') AS UPDATED, DP.UPDATED_BY, DP.DEPLOY_DATE, DP.STATUS,
                           DECODE(CT.DSARVOREPROD, NULL, PROD.NMPRODUTO, CT.DSARVOREPROD) AS NMPRODUTO, DP.PARENT,
                           DP.IS_CUSTOMER_DEPLOY, OP.NRPARCNEGOCIO AS NRINTERLOCUTORNEGOCIO, PN.NMPRINCIPALPARC,
                           DECODE(DP.PARENT, NULL, DP.ID || ' - ' || DP.NAME, PDP.ID || ' - ' || PDP.NAME) AS DEPLOY_PARENT_NAME,
                           OP.NMOPERADOR AS NMPARCINTERLOCUTOR, OP.DSEMAILOPER AS EMAIL, CT.*
                      FROM DEPLOY DP
                 LEFT JOIN DEPLOY PDP                ON PDP.ID = DP.PARENT
                 LEFT JOIN PRODUCT_VERSION PV        ON PV.ID = DP.PRODUCT_VERSION_ID
                 LEFT JOIN CONTATO CT                ON DP.TICKET_ID = CT.NRSEQCONTATO
                 LEFT JOIN ZH_USER RC                ON RC.ID = DP.CREATED_BY
                 LEFT JOIN ZH_USER RU                ON RU.ID = DP.UPDATED_BY
                 LEFT JOIN OPERADOR OP               ON OP.CDOPERADOR = CT.CDOPERINCLUSAO
                 LEFT JOIN PARCNEGOCIO PN            ON PN.NRPARCNEGOCIO = CT.NRPARCNEGOCIO
                 LEFT JOIN DEPLOY_SERVER DS          ON DP.ID = DS.DEPLOY_ID
                 LEFT JOIN PRODUCT_SERVER PS         ON PS.ID = DS.PRODUCT_SERVER_ID
                 LEFT JOIN SERVER S                  ON S.ID = PS.SERVER_ID
                 LEFT JOIN PRODUCT P                 ON P.ID = DP.PRODUCT_ID
                 LEFT JOIN PRODUTO PROD              ON CT.CDPRODUTO = PROD.CDPRODUTO
                                                    AND CT.NRORG = PROD.NRORG
                 LEFT JOIN RESOURCE_T R              ON R.ID = S.RESOURCE_T_ID
                 LEFT JOIN CLIENTE CLI               ON CLI.CDCLIENTE = R.EXTERNAL_CODE
                 LEFT JOIN CRM_CLASSIFICACAOCASO CCC ON CCC.NRCLASSIFICACAOCASO = CT.NRCLASSIFICACAOCASO
                 LEFT JOIN NATUCONT NC               ON NC.CDNATUCONT = CT.CDNATUCONT
                 LEFT JOIN MOTICONT MC               ON MC.CDMOTICONT = CT.CDMOTICONT
                     WHERE DP.DEPLOY_DATE IS NOT NULL
                       AND DP.IS_ACTIVE = 'Y'
                       AND DP.STATUS = 'D')
             WHERE 1 = 1
             $conditions
          ORDER BY DEPLOY_DATE DESC, PARENT DESC NULLS FIRST, DEPLOY_ID
SQL;
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    public function getResources($conditions, $limit, $offset){
        $query = <<<SQL
            SELECT * FROM(
                SELECT ID AS CREATED_BY, NAME AS CREATED_BY_NAME FROM ZH_USER
            ) WHERE 1=1 $conditions
SQL;
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    public function getProducts($conditions, $limit, $offset){
        $query = <<<SQL
            SELECT * FROM(
                SELECT ID,NAME,DESCRIPTION FROM PRODUCT
            ) WHERE 1=1 $conditions
SQL;
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    public function getServers($conditions, $limit, $offset){
        $query = <<<SQL
            SELECT * FROM(
                SELECT ID,NAME FROM SERVER
            ) WHERE 1=1 $conditions
SQL;
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }
    
    public function getParcNegocio($conditions, $limit, $offset, $nrorg){
        $query = <<<SQL
            SELECT * FROM(
                SELECT * FROM PARCNEGOCIO
                 WHERE NRORG = :P_NRORG AND IDATIVO = 'S' AND CDTIPOPARCPRINCIPAL IN ('INTERLOCUTOR', 'OPERADOR')
            ) WHERE 1=1 $conditions
SQL;
        $params = array("P_NRORG" => $nrorg);
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getNatuCont($cdOperador,$nrorg){
        $query = <<<SQL
            SELECT DISTINCT NATUCONT.CDNATUCONT, NATUCONT.NMNATUCONT, NATUCONT.NRORG
              FROM NATUCONT
        INNER JOIN CRM_PERFINATUCONT ON NATUCONT.CDNATUCONT = CRM_PERFINATUCONT.CDNATUCONT AND NATUCONT.NRORG = CRM_PERFINATUCONT.NRORG
        INNER JOIN CRM_PERFIOPER ON CRM_PERFINATUCONT.NRSEQPERFATIV = CRM_PERFIOPER.NRSEQPERFATIV AND CRM_PERFINATUCONT.NRORG = CRM_PERFIOPER.NRORG
             WHERE (CRM_PERFIOPER.CDOPERADOR = :P_CDOPERADOR AND NATUCONT.NRORG = :P_NRORG AND NATUCONT.IDATIVO = 'S')
SQL;
        $params = array(
            "P_CDOPERADOR" => $cdOperador,
            "P_NRORG" => $nrorg
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query,$params);
    }
    
    public function getMoticont($conditions,$cdProduto){
        $query = <<<SQL
            SELECT MOTI.*, PROD.NMPRODUTO, PROD.CDPRODUTO
            FROM MOTICONT MOTI
            LEFT JOIN CRM_RELACMOTIPROD MOTIPROD ON MOTIPROD.NRORG = MOTI.NRORG AND MOTI.CDMOTICONT = MOTIPROD.CDMOTICONT
            LEFT JOIN PRODUTO PROD ON PROD.NRORG = MOTI.NRORG AND PROD.CDPRODUTO = MOTIPROD.CDPRODUTO
            WHERE EXISTS ( SELECT 1 FROM CRM_RELACMOTIPROD R WHERE R.CDPRODUTO = PROD.CDPRODUTO AND R.CDMOTICONT = MOTI.CDMOTICONT) AND PROD.CDPRODUTO IN( :CDPRODUTO )
SQL;
        $params = array(
            "CDPRODUTO" => $cdProduto
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query,$params);
    }

    public function getClassificacaoCaso($nrorg,$conditions){
        $query = <<<SQL
            SELECT * FROM (
                        SELECT CLC.CDNATUCONT, CLAS.*, ALRT.DSALERTACRM 
                          FROM CRM_CLASSIFICACAOCASO CLAS 
                     LEFT JOIN CRM_ALERTAS ALRT ON CLAS.NRORG = ALRT.NRORG AND CLAS.NRALERTACRM = ALRT.NRALERTACRM
                          JOIN CRM_NATUCONTCLASSIFICACASO CLC ON CLC.NRCLASSIFICACAOCASO = CLAS.NRCLASSIFICACAOCASO)
            WHERE NRORG = :P_NRORG AND IDATIVO = 'S' $conditions
SQL;
        $params = array(
            "P_NRORG" => $nrorg
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query,$params);
    }

    public function getServicoProdutoReferencia($cdNatuCont,$nrClassificacaoCaso,$conditions){
        $query = <<<SQL
        SELECT DISTINCT * FROM (
        SELECT * FROM (
            SELECT 'A' AS TIPO, NULL AS PARENT, TO_CHAR(TA.NRTPARVPROD) AS ID, TA.DSTPARVPROD AS NAME,
                   TA.NRTPARVPROD, CC.CHILDCOUNT,
                   CASE WHEN CHILDCOUNT > 0 THEN 'S' ELSE 'N' END AS HASCHILD, TA.NRORG, '' AS CDPRODUTO FROM TIPOARVOREPROD TA,
                   (
                    SELECT TA.NRTPARVPROD, COUNT(EA.NRTPARVPROD) AS CHILDCOUNT
                    FROM ESTRUARVOREPROD EA, TIPOARVOREPROD TA
                    WHERE TA.NRTPARVPROD = EA.NRTPARVPROD GROUP BY TA.NRTPARVPROD
                   ) CC
             WHERE CC.NRTPARVPROD = TA.NRTPARVPROD
             AND (INSTR(',' ||:NRTPARVPROD ||',', ',' ||TA.NRTPARVPROD ||',') > 0 OR :NRTPARVPROD IS NULL) 
             
             UNION ALL 
             
             SELECT 'E' AS TIPO, TO_CHAR(EA.NRTPARVPROD) || TO_CHAR(NVL(EA.NRARVOREPRODPAI, '')) AS PARENT,
                     TO_CHAR(EA.NRTPARVPROD) || TO_CHAR(EA.NRARVOREPROD) AS ID, EA.NMARVOREPROD AS NAME,
                     EA.NRTPARVPROD, CC.CHILDCOUNT, CASE WHEN CHILDCOUNT > 0 THEN 'S' ELSE 'N' END AS HASCHILD,
                     EA.NRORG, '' AS CDPRODUTO FROM ESTRUARVOREPROD EA, 
                     (
                        SELECT EA.NRARVOREPROD, COUNT(PA.NRARVOREPROD) AS CHILDCOUNT
                        FROM PRODARVOREPROD PA, ESTRUARVOREPROD EA
                        WHERE EA.NRARVOREPROD = PA.NRARVOREPROD(+)
                        GROUP BY EA.NRARVOREPROD 
                     ) CC
             WHERE EA.NRARVOREPROD = CC.NRARVOREPROD
             AND (INSTR(',' ||:NRTPARVPROD ||',', ',' ||EA.NRTPARVPROD ||',') > 0 OR :NRTPARVPROD IS NULL)
             
             UNION ALL
             
             SELECT 'P' AS TIPO, TO_CHAR(PA.NRTPARVPROD) || TO_CHAR(PA.NRARVOREPROD) AS PARENT,
             PA.CDPRODUTO AS ID, PD.NMPRODUTO AS NAME, PA.NRTPARVPROD, 0 AS CHILDCOUNT, 'N' AS HASCHILD,
             PA.NRORG, PA.CDPRODUTO
             FROM PRODARVOREPROD PA, PRODUTO PD
             WHERE PA.CDPRODUTO = PD.CDPRODUTO
             AND (INSTR(',' ||:NRTPARVPROD ||',', ',' ||PA.NRTPARVPROD ||',') > 0 OR :NRTPARVPROD IS NULL) 
        ) WHERE NVL(PARENT, -1) != ID) RES 
          CONNECT BY PRIOR RES.PARENT = RES.ID
          START WITH RES.CDPRODUTO IN (
                                    SELECT CDPRODUTO
                                    FROM CRM_CLASSIFICACASOPROD
                                    WHERE NRORG = RES.NRORG
                                    AND CDNATUCONT IN (:CDNATUCONT)
                                    AND NRCLASSIFICACAOCASO IN (:NRCLASSIFICACAOCASO) 
                                      ) OR (
                                    RES.CDPRODUTO IS NOT NULL AND (:CDNATUCONT IS NULL OR :NRCLASSIFICACAOCASO IS NULL)
                                    ) $conditions
 
SQL;
        $params = array(
            "NRTPARVPROD" => null,
            "NRCLASSIFICACAOCASO" => $nrClassificacaoCaso,
            "CDNATUCONT" => $cdNatuCont
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query,$params);
    }

    public function obtemTodos($conditions, $limit, $offset){
        $query = <<<SQL
		SELECT *
		  FROM ( SELECT PROD.ID, PROD.NMPRODUTO, PROD.DSURL_JSON_MENU, PROD.IDSTATUS,
    		            PROD.DTINCLUSAO, PROD.DTULTATU, PROD.NREXTERNAL_PRODUCT_ID,
    		            EXPROJ.PRODUCT_TYPE AS PM_PRODUCT_TYPE,
    		            PROD.IDMODOPROC, PROD.IDTPLINHAPRODUTO, PROD.DSLINHAPRODUTO,
    		            TPMED.CDTPMEDIDASERV, TPMED.DSTPMEDIDASERV,
    		            CASE PROD.IDPRODUTOSAAS
    		            WHEN :SAAS        THEN 'Exclusivamente SAAS'
                        WHEN :NAOSAAS     THEN 'Exclusivamente NÃO SAAS'
                        WHEN :ORGANIZACAO THEN 'Assume identificador da Organização'
                         END AS IDPRODUTOSAAS
    		       FROM OM_PRODUTO PROD
	         INNER JOIN PRODUCT EXPROD ON EXPROD.ID = PROD.NREXTERNAL_PRODUCT_ID
	         INNER JOIN PROJECT EXPROJ ON EXPROJ.ID = EXPROD.PROJECT_ID
	          LEFT JOIN FCT_TIPOMEDIDASERV TPMED ON TPMED.CDTPMEDIDASERV = PROD.CDTPMEDIDASERV)
		 WHERE 1=1
		      $conditions
      ORDER BY NMPRODUTO
SQL;
                    $param = array(
                        'SAAS' => Entities\OmProduto::PRODUTO_SAAS,
                        'NAOSAAS' => Entities\OmProduto::PRODUTO_NAO_SAAS,
                        'ORGANIZACAO' => Entities\OmProduto::HERDA_ORGANIZACAO,
                    );
        $query  = Utilities::getPaginatedQuery($query, $limit, $offset);
        $organizacoes = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        foreach($organizacoes as &$organizacao){
            $organizacao['ID'] = intval($organizacao['ID']);
        }
        return $organizacoes;
    }

    public function getProduct($productId){
        $query = <<<SQL
		   SELECT PROD.ID, PROD.NMPRODUTO, PROD.DSURL_JSON_MENU, PROD.IDSTATUS,
		          PROD.DTINCLUSAO, PROD.DTULTATU, OPERINC.NMOPERADOR AS NMOPERINCLUSAO,
		          OPERATU.NMOPERADOR AS NMOPERULTATU, PROD.NREXTERNAL_PRODUCT_ID,
		          PROD.CDAPPNOTIF, PROD.DSCHAVENOTIF, PROD.IDPRODUTOSAAS,
    		      EXPROJ.PRODUCT_TYPE AS PM_PRODUCT_TYPE
	 	     FROM OM_PRODUTO PROD
		LEFT JOIN OPERADOR OPERINC ON (OPERINC.CDOPERADOR = PROD.CDOPERINCLUSAO)
		LEFT JOIN OPERADOR OPERATU ON (OPERATU.CDOPERADOR = PROD.CDOPERULTATU)
	   INNER JOIN PRODUCT EXPROD   ON EXPROD.ID = PROD.NREXTERNAL_PRODUCT_ID
	   INNER JOIN PROJECT EXPROJ   ON EXPROJ.ID = EXPROD.PROJECT_ID
	     	WHERE PROD.ID = $productId
SQL;
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    public function obtemMaiorId(){
        $query = "SELECT MAX(TO_NUMBER(ID))+1 AS MAX_ID FROM OM_PRODUTO";
        $result = $this->getEntityManager()->getConnection()->fetchAll($query);
        return $result[0]['MAX_ID'] ? $result[0]['MAX_ID'] : 1;
    }

    public function filterDevOpsProducts($productIds) {
        $query = <<<SQL
		SELECT PD.ID, PD.NAME, PD.ORGANIZATION_ID, PD.IS_ACTIVE
          FROM PRODUCT PD
    INNER JOIN PROJECT PJ ON PJ.ID = PD.PROJECT_ID
                         AND PJ.PRODUCT_TYPE IN ('C', 'I')

         WHERE PD.ID IN (:PRODUCT_IDS)
SQL;
        $params     = array( 'PRODUCT_IDS' => $productIds );
        $paramTypes = array( 'PRODUCT_IDS' => Connection::PARAM_STR_ARRAY );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    /**
     * @param $devOpsId
     * @return array
     */
    public function getProductLanguages($devOpsId){
        $query = "SELECT ZL.ID, LOWER(ZL.SHORT_NAME) AS SHORT_NAME,
                         ZL.LANGUAGE_ISO, ZL.COUNTRY_CODE, ZL.NAME
                    FROM PRODUCT PR
                    JOIN PROJECT PJ ON PJ.ID = PR.PROJECT_ID
                    JOIN ZH_PROJECT_REPOSITORY RT ON RT.PROJECT_ID = PJ.ID
                    JOIN ZH_REPOSITORY RP ON RP.ID = RT.REPOSITORY_ID
                    JOIN ZH_REPOSITORY_LANGUAGE ZR ON ZR.REPOSITORY_ID = RP.ID
                    JOIN ZH_LANGUAGE ZL ON ZL.ID = ZR.LANGUAGE_ID
                   WHERE PR.ID = :PRODUCT_ID
                     AND RT.IS_ACTIVE = 'Y'
                     AND ZL.IS_ACTIVE = 'Y'
                GROUP BY ZL.ID, ZL.SHORT_NAME, ZL.LANGUAGE_ISO, ZL.COUNTRY_CODE, ZL.NAME
                ORDER BY ZL.NAME";

        $params = array( 'PRODUCT_ID' => $devOpsId );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $devOpsId
     * @return array
     */
    public function getProductDefaultLanguage($devOpsId){
        $query = "SELECT ZL.ID, LOWER(ZL.SHORT_NAME) AS SHORT_NAME,
                         ZL.LANGUAGE_ISO, ZL.COUNTRY_CODE, ZL.NAME
                    FROM PRODUCT PR
                    JOIN PROJECT PJ ON PJ.ID = PR.PROJECT_ID
                    JOIN ZH_PROJECT_REPOSITORY RT ON RT.PROJECT_ID = PJ.ID
                    JOIN ZH_REPOSITORY RP ON RP.ID = RT.REPOSITORY_ID
                    JOIN ZH_LANGUAGE ZL ON ZL.ID = RP.DEFAULT_LANGUAGE
                    JOIN REPOSITORY RS ON RS.URL = RP.URL
                                      AND RS.PROJECT = PJ.ID
                   WHERE PR.ID = :PRODUCT_ID
                     AND RT.IS_ACTIVE = 'Y'
                     AND ZL.IS_ACTIVE = 'Y'
                     AND RS.STATUS = 'A'
                GROUP BY ZL.ID, ZL.SHORT_NAME, ZL.LANGUAGE_ISO, ZL.COUNTRY_CODE, ZL.NAME
                ORDER BY ZL.NAME";

        $params = array( 'PRODUCT_ID' => $devOpsId );
        $registers = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return empty($registers) ? $registers : $registers[0];
    }
}