<?php

namespace Teknisa\ACL\Services;

use Teknisa\ACL\Repositories\PerfAcessParc as PerfAcessParcRepository;
use Teknisa\ACL\Util\InstanceProvider;
use Teknisa\Libs\Util\Environment;
use Teknisa\ACL\Util\AclUtilities;

class PerfAcessParc {

    const MAXNRPERFACESSPARC = "MAXNRPERFACESSPARC";
    const DTVALPERFILPARC = "DTVALPERFILPARC";
    const IDATIVO = "IDATIVO";
    const YES = "S";
    const NRPERFACESSPARC = "NRPERFACESSPARC";
    const NRORG = "NRORG";
    const NRORGINCLUSAO = "NRORGINCLUSAO";
    const NRORGULTATU = "NRORGULTATU";
    const DTINCLUSAO = "DTINCLUSAO";
    const DTULTATU = "DTULTATU";
    const CDOPERINCLUSAO = "CDOPERINCLUSAO";
    const CDOPERULTATU = "CDOPERULTATU";

    /** @var PerfAcessParc */
    protected $perfAcessParcRepository;

    /** @var Environment */
    protected $environment;

    public function __construct(PerfAcessParcRepository $perfAcessParcRepository, Environment $environment) {
        $this->perfAcessParcRepository = $perfAcessParcRepository;
        $this->environment = $environment;
    }

    public function getMaxNrPerfilAcesso() {
        return InstanceProvider::getNewCode()->getCodeByTableName('PERFACESSPARC', $this->environment->getCurrentOrganizationId());
    }

    public function setDefaultValues(&$row) {
        $this->setNrOrgValues($row);
        $this->setDateValues($row);
        $this->setOperatorValues($row);
        $row[self::DTVALPERFILPARC] = null;
        $row[self::IDATIVO] = self::YES;
    }

    protected function setNrOrgValues(&$row) {
        $nrOrg = $this->getNrOrgFilter();
        $row[self::NRORG] = $this->setValueIfDontExist($row, self::NRORG, $nrOrg);
        $row[self::NRORGINCLUSAO] = $this->setValueIfDontExist($row, self::NRORGINCLUSAO, $nrOrg);
        $row[self::NRORGULTATU] = $nrOrg;
    }

    protected function setDateValues(&$row) {
        $time = new \DateTime();
        $timeFormatted = $time->format('d/m/Y');
        $row[self::DTINCLUSAO] = $this->setValueIfDontExist($row, self::DTINCLUSAO, $timeFormatted);
        $row[self::DTULTATU] = $timeFormatted;
    }

    protected function setOperatorValues(&$row) {
        $cdOperador = $this->environment->getCurrentUserId();
        $row[self::CDOPERINCLUSAO] = $this->setValueIfDontExist($row, self::CDOPERINCLUSAO, $cdOperador);
        $row[self::CDOPERULTATU] = $cdOperador;
    }

    protected function setValueIfDontExist($row, $property, $value) {
        return isset($row[$property]) ? $row[$property] : $value;
    }

    public function getNrOrgFilter() {
        return $this->environment->getCurrentOrganizationId();
    }

    public function findPerfAcessParcOperador($conditions, $orderBy)
    {
        $orderByString = '';
        $filters = AclUtilities::createQueryFiltered($conditions);
        foreach($orderBy as $idx => $order) {
            if($idx === 'DTVALIDADE'){
                $orderByString .= ' ORDER BY TO_DATE(' . $idx . ') ' . $order . ' NULLS LAST';
            } else {
                $orderByString .= ' ORDER BY ' . $idx . ' ' . $order;
            }
        }
        $data = InstanceProvider::getRepository()->getPerfAcessParc()->findPerfAcessParcOperador($filters, $orderByString);
        foreach ($data as &$p){
            if(!$p['DTVALIDADE']){
                $p['DTVALIDADE'] = '';
            }
        }
        return $data;
    }

}